#!/home/lukas/dev/COVID-19/csse_covid_19_data/csse_covid_19_daily_reports/venv/bin/python3
import os
import datetime
import csv

# INSTALL:
# git clone https://github.com/CSSEGISandData/COVID-19.git
# git clone https://github.com/pcm-dpc/COVID-19 pcm-COVID-19/
# copy this script to COVID-19/csse_covid_19_data/csse_covid_19_daily_reports$
# python3 -m venv venv; source venv/bin/activate
# pip install plotly request beautifulsoup4 pytz
# mkdir -p temps
# python3 fetch_temps.py
# python3 per_country.py

# external deps - plotly
from plotly import offline
import plotly.graph_objects as go

ITALY_GIT = os.getcwd() + "/../../../pcm-COVID-19"

rate_cutoff = 1.5  # do not show R7 until daily 7 day moving average new cases per million reach at least this value

# for easier plotting ... the rate Y axis can be limited to this value,
rate_cap = 10

fns = [fn for fn in os.listdir() if fn.endswith(".csv")]
def key(fn):
    fn = fn.split("-")
    return fn[2]+fn[0]+fn[1]
fns.sort(key=key)
fns_italy_regions = [fn for fn in os.listdir(ITALY_GIT+"/dati-regioni")
    if fn.endswith(".csv") and not fn.endswith("regioni.csv") and not fn.endswith("regioni-latest.csv")]
fns_italy_regions.sort()

##### copy over to per_country_index.py and per_country_rate.py
countries = [
    "USA",  # states in us_countries
    "Canada",  # provinces set in canada_countries
    "EU",  # eu countries in eu_countries
    # Europe
    "UK", "Serbia", "Albania", "Russia", "Ukraine", "Belarus", "Switzerland", "Moldova",
    "Bosnia and Herzegovina", "Iceland", "Faroe Islands",
    # APAC
    "Guam (US)", "Singapore", "Taiwan", "New Zealand", "Japan", "South Korea", "Cambodia",
    "Vietnam", "Malaysia", "Thailand", "Laos", "Philippines", "Indonesia", "Papua New Guinea", "Australia",
    # Africa
    "Seychelles", "Egypt", "Senegal", "Burkina Faso", "Nigeria", "Morocco",
    "Tunisia", "Mali", "Kenya", "South Africa", "Isle of Man",
    # Latin America
    "Peru", "Mexico", "Cuba", "Guatemala", "Costa Rica", "Brazil", "Bolivia", "Chile",
    "Argentina", "Uruguay", "Paraguay", "Colombia",
    # Middle East and Central Asia
    "Kazakhstan", "Uzbekistan", "Kyrgyzstan", "Mongolia", "Iraq", "Iran", "Saudi Arabia", "Jordan", "Israel", "Turkey",
    # Others
    "India", "Pakistan", "Nepal", "Falkland Islands", "Greenland",
]

eu_countries = [
    "Czechia", "Netherlands", "Italy", "Slovakia", "Ireland", "Denmark", "Spain", "Luxembourg", "Slovenia",
    "Greece", "Sweden", "Austria", "Belgium", "Bulgaria", "Croatia", "Cyprus", "Estonia", "Finland", "France",
    "Germany", "Hungary", "Latvia", "Lithuania", "Malta", "Poland", "Portugal", "Romania",
]
countries.extend(eu_countries)

us_countries = ["Alabama", "Alaska", "Arizona", "Arkansas", "California", "Colorado", "Connecticut", "Delaware",
    "Florida", "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana", "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
    "Maryland", "Massachusetts", "Michigan", "Minnesota", "Mississippi", "Missouri", "Montana", "Nebraska", "Nevada", "New Hampshire",
    "New Jersey", "New Mexico", "New York", "North Carolina", "North Dakota", "Ohio", "Oklahoma", "Oregon", "Pennsylvania",
    "Rhode Island", "South Carolina", "South Dakota", "Tennessee", "Texas", "Utah", "Vermont", "Virginia", "Washington", "West Virginia",
    "Wisconsin", "Wyoming"]
countries.extend(us_countries)

canada_countries = ["Alberta", "British Columbia", "Manitoba", "New Brunswick", "Newfoundland and Labrador", "Northwest Territories",
    "Nova Scotia", "Ontario", "Prince Edward Island", "Quebec", "Saskatchewan", "Yukon"]
countries.extend(canada_countries)

italy_regions = [
"Abruzzo (Italy region)", "Basilicata (Italy region)", 
"P.A. Bolzano (Italy region)",  # south tyrol
"Calabria (Italy region)", "Campania (Italy region)", "Emilia-Romagna (Italy region)", "Friuli Venezia Giulia (Italy region)",
"Lazio (Italy region)", "Liguria (Italy region)", "Lombardia (Italy region)", "Marche (Italy region)", "Molise (Italy region)",
"Piemonte (Italy region)", "Puglia (Italy region)", "Sardegna (Italy region)", "Sicilia (Italy region)", "Toscana (Italy region)",
"P.A. Trento (Italy region)", "Umbria (Italy region)", "Valle d'Aosta (Italy region)", "Veneto (Italy region)",
]
countries.extend(italy_regions)

DISPLAY_NAME = {
    "Czechia": "Czech Republic",
    "Guam (US)": "Guam",
}
for c in us_countries:
    if c == "New York":
        DISPLAY_NAME[c] = "New York (state) (US)"
    else:
        DISPLAY_NAME[c] = c + " (US)"
for c in canada_countries:
    DISPLAY_NAME[c] = c + " (Canada)"
##### ^^ end of countries and DISPLAY_NAME part

ALTERNATIVE_NAMES = {
    "Belgium": [
        ("Antwerp","Belgium"), ("Brussels","Belgium"), ("East Flanders","Belgium"), ("Flemish Brabant","Belgium"),
        ("Hainaut","Belgium"), ("Liege","Belgium"), ("Limburg","Belgium"), ("Luxembourg","Belgium"), ("Namur","Belgium"),
        ("Unknown","Belgium"), ("Walloon Brabant","Belgium"), ("West Flanders","Belgium"),
    ],
    "Czechia": ["Czech Republic"],
    "Taiwan": ["Taiwan*", ("Taiwan","Taipei and environs"), ("Taiwan","Taiwan")],
    "Germany": [("Bavaria","Germany"), ("Baden-Wurttemberg","Germany"), ("Bayern","Germany"),
        ("Berlin","Germany"),("Brandenburg","Germany"), ("Bremen","Germany"), ("Hamburg","Germany"),
        ("Hessen","Germany"), ("Mecklenburg-Vorpommern","Germany"), ("Niedersachsen","Germany"),
        ("Nordrhein-Westfalen","Germany"), ("Rheinland-Pfalz","Germany"), ("Saarland","Germany"),
        ("Sachsen","Germany"), ("Sachsen-Anhalt","Germany"), ("Schleswig-Holstein","Germany"),
        ("Thuringen","Germany"), ("Unknown","Germany")],
    "Italy": [("Abruzzo","Italy"), ("Basilicata","Italy"), ("Calabria","Italy"), ("Campania","Italy"),
        ("Emilia-Romagna","Italy"), ("Friuli Venezia Giulia","Italy"), ("Lazio","Italy"), ("Liguria","Italy"),
        ("Lombardia","Italy"), ("Marche","Italy"), ("Molise","Italy"), ("P.A. Bolzano","Italy"), ("P.A. Trento","Italy"),
        ("Piemonte","Italy"), ("Puglia","Italy"), ("Sardegna","Italy"), ("Sicilia","Italy"), ("Toscana","Italy"),
        ("Umbria","Italy"), ("Valle d'Aosta","Italy"), ("Veneto","Italy")],
    "Spain": [("Andalusia","Spain"), ("Aragon","Spain"), ("Asturias","Spain"), ("Baleares","Spain"), ("C. Valenciana","Spain"),
        ("Canarias","Spain"), ("Cantabria","Spain"), ("Castilla - La Mancha","Spain"), ("Castilla y Leon","Spain"),
        ("Catalonia","Spain"), ("Ceuta","Spain"), ("Extremadura","Spain"), ("Galicia","Spain"), ("La Rioja","Spain"),
        ("Madrid","Spain"), ("Melilla","Spain"), ("Murcia","Spain"), ("Navarra","Spain"), ("Pais Vasco","Spain")],
    "South Korea": ["Korea, South", "Republic of Korea"],
    "Ireland": ["Republic of Ireland"],
    "Iran": ["Iran (Islamic Republic of)"],
    "Guam (US)": [("Guam", "US")],
    "Denmark": [("Denmark", "Denmark")],
    "France": [("France", "France")],
    "Faroe Islands": [("Faroe Islands", "Denmark")],
    "Falkland Islands": [("Falkland Islands (Islas Malvinas)","United Kingdom"),
                         ("Falkland Islands (Malvinas)","United Kingdom")],
    "Greenland": [("Greenland", "Denmark")],
    "Isle of Man": [("Isle of Man", "United Kingdom")],
    "Vietnam": ["Viet Nam"],
    "Moldova": ["Republic of Moldova"],
    "UK": [
        ("UK", "United Kingdom"), ("United Kingdom", "United Kingdom"), "United Kingdom",
        ("England","United Kingdom"), ("Northern Ireland","United Kingdom"),
        ("Scotland","United Kingdom"), ("Unknown","United Kingdom"), ("Wales","United Kingdom"),
    ],
    "Australia": [("Australian Capital Territory","Australia"), ("New South Wales","Australia"),
        ("Northern Territory","Australia"), ("Queensland","Australia"), ("South Australia","Australia"),
        ("Tasmania","Australia"), ("Victoria","Australia"), ("Western Australia","Australia"),
    ],
    "Brazil": [
        ("Acre","Brazil"), ("Alagoas","Brazil"), ("Amapa","Brazil"), ("Amazonas","Brazil"),
        ("Bahia","Brazil"), ("Ceara","Brazil"), ("Distrito Federal","Brazil"), ("Espirito Santo","Brazil"),
        ("Goias","Brazil"), ("Maranhao","Brazil"), ("Mato Grosso","Brazil"), ("Mato Grosso do Sul","Brazil"), ("Minas Gerais","Brazil"),
        ("Para","Brazil"), ("Paraiba","Brazil"), ("Parana","Brazil"), ("Pernambuco","Brazil"), ("Piaui","Brazil"),
        ("Rio Grande do Norte","Brazil"), ("Rio Grande do Sul","Brazil"), ("Rio de Janeiro","Brazil"), ("Rondonia","Brazil"),
        ("Roraima","Brazil"), ("Santa Catarina","Brazil"), ("Sao Paulo","Brazil"), ("Sergipe","Brazil"), ("Tocantins","Brazil"),
        ("Unknown","Brazil"),
    ],
    "Chile": [
        ("Antofagasta","Chile"), ("Araucania","Chile"), ("Arica y Parinacota","Chile"), ("Atacama","Chile"), ("Aysen","Chile"),
        ("Biobio","Chile"), ("Coquimbo","Chile"), ("Los Lagos","Chile"), ("Los Rios","Chile"), ("Magallanes","Chile"), ("Maule","Chile"),
        ("Metropolitana","Chile"), ("Nuble","Chile"), ("OHiggins","Chile"), ("Tarapaca","Chile"), ("Unknown","Chile"), ("Valparaiso","Chile"),
    ],
    "Mexico": [
        ("Aguascalientes","Mexico"), ("Baja California","Mexico"), ("Baja California Sur","Mexico"), ("Campeche","Mexico"), ("Chiapas","Mexico"),
        ("Chihuahua","Mexico"), ("Ciudad de Mexico","Mexico"), ("Coahuila","Mexico"), ("Colima","Mexico"), ("Durango","Mexico"), ("Guanajuato","Mexico"),
        ("Guerrero","Mexico"), ("Hidalgo","Mexico"), ("Jalisco","Mexico"), ("Mexico","Mexico"), ("Michoacan","Mexico"), ("Morelos","Mexico"), ("Nayarit","Mexico"),
        ("Nuevo Leon","Mexico"), ("Oaxaca","Mexico"), ("Puebla","Mexico"), ("Queretaro","Mexico"), ("Quintana Roo","Mexico"), ("San Luis Potosi","Mexico"),
        ("Sinaloa","Mexico"), ("Sonora","Mexico"), ("Tabasco","Mexico"), ("Tamaulipas","Mexico"), ("Tlaxcala","Mexico"), ("Veracruz","Mexico"), ("Yucatan","Mexico"),
        ("Zacatecas","Mexico"),
    ],
    "Peru": [
        ("Amazonas","Peru"), ("Ancash","Peru"), ("Apurimac","Peru"), ("Arequipa","Peru"), ("Ayacucho","Peru"), ("Cajamarca","Peru"), ("Callao","Peru"), ("Cusco","Peru"),
        ("Huancavelica","Peru"), ("Huanuco","Peru"), ("Ica","Peru"), ("Junin","Peru"), ("La Libertad","Peru"), ("Lambayeque","Peru"), ("Lima","Peru"), ("Loreto","Peru"),
        ("Madre de Dios","Peru"), ("Moquegua","Peru"), ("Pasco","Peru"), ("Piura","Peru"), ("Puno","Peru"), ("San Martin","Peru"), ("Tacna","Peru"), ("Tumbes","Peru"),
        ("Ucayali","Peru"), ("Unknown","Peru"),
    ],
    "Japan": [
        ("Aichi","Japan"), ("Akita","Japan"), ("Aomori","Japan"), ("Chiba","Japan"), ("Ehime","Japan"), ("Fukui","Japan"), ("Fukuoka","Japan"), ("Fukushima","Japan"),
        ("Gifu","Japan"), ("Gunma","Japan"), ("Hiroshima","Japan"), ("Hokkaido","Japan"), ("Hyogo","Japan"), ("Ibaraki","Japan"), ("Ishikawa","Japan"),
        ("Kagawa","Japan"), ("Kagoshima","Japan"), ("Kanagawa","Japan"), ("Kochi","Japan"), ("Kumamoto","Japan"), ("Kyoto","Japan"), ("Mie","Japan"), ("Miyagi","Japan"),
        ("Miyazaki","Japan"), ("Nagano","Japan"), ("Nagasaki","Japan"), ("Nara","Japan"), ("Niigata","Japan"), ("Oita","Japan"), ("Okayama","Japan"), ("Okinawa","Japan"),
        ("Osaka","Japan"), ("Port Quarantine","Japan"), ("Saga","Japan"), ("Saitama","Japan"), ("Shiga","Japan"), ("Shimane","Japan"), ("Shizuoka","Japan"), ("Tochigi","Japan"),
        ("Tokushima","Japan"), ("Tokyo","Japan"), ("Tottori","Japan"), ("Toyama","Japan"), ("Unknown","Japan"), ("Wakayama","Japan"), ("Yamagata","Japan"), ("Yamaguchi","Japan"),
        ("Yamanashi","Japan"),
    ],
    "Colombia": [
        ("Amazonas","Colombia"), ("Antioquia","Colombia"), ("Arauca","Colombia"), ("Atlantico","Colombia"), ("Bolivar","Colombia"), ("Boyaca","Colombia"),
        ("Caldas","Colombia"), ("Capital District","Colombia"), ("Caqueta","Colombia"), ("Casanare","Colombia"), ("Cauca","Colombia"), ("Cesar","Colombia"),
        ("Choco","Colombia"), ("Cordoba","Colombia"), ("Cundinamarca","Colombia"), ("Guainia","Colombia"), ("Huila","Colombia"), ("La Guajira","Colombia"),
        ("Magdalena","Colombia"), ("Meta","Colombia"), ("Narino","Colombia"), ("Norte de Santander","Colombia"), ("Putumayo","Colombia"), ("Quindio","Colombia"),
        ("Risaralda","Colombia"), ("San Andres y Providencia","Colombia"), ("Santander","Colombia"), ("Sucre","Colombia"), ("Tolima","Colombia"), ("Unknown","Colombia"),
        ("Valle del Cauca","Colombia"), ("Vaupes","Colombia"), ("Vichada","Colombia"),
    ],
    "Russia": [
        "Russian Federation",
        ("Adygea Republic","Russia"), ("Altai Krai","Russia"), ("Altai Republic","Russia"), ("Amur Oblast","Russia"), ("Arkhangelsk Oblast","Russia"),
        ("Astrakhan Oblast","Russia"), ("Bashkortostan Republic","Russia"), ("Belgorod Oblast","Russia"), ("Bryansk Oblast","Russia"), ("Buryatia Republic","Russia"),
        ("Chechen Republic","Russia"), ("Chelyabinsk Oblast","Russia"), ("Chukotka Autonomous Okrug","Russia"), ("Chuvashia Republic","Russia"), ("Dagestan Republic","Russia"),
        ("Ingushetia Republic","Russia"), ("Irkutsk Oblast","Russia"), ("Ivanovo Oblast","Russia"), ("Jewish Autonomous Okrug","Russia"), ("Kabardino-Balkarian Republic","Russia"),
        ("Kaliningrad Oblast","Russia"), ("Kalmykia Republic","Russia"), ("Kaluga Oblast","Russia"), ("Kamchatka Krai","Russia"), ("Karachay-Cherkess Republic","Russia"),
        ("Karelia Republic","Russia"), ("Kemerovo Oblast","Russia"), ("Khabarovsk Krai","Russia"), ("Khakassia Republic","Russia"), ("Khanty-Mansi Autonomous Okrug","Russia"),
        ("Kirov Oblast","Russia"), ("Komi Republic","Russia"), ("Kostroma Oblast","Russia"), ("Krasnodar Krai","Russia"), ("Krasnoyarsk Krai","Russia"), ("Kurgan Oblast","Russia"),
        ("Kursk Oblast","Russia"), ("Leningrad Oblast","Russia"), ("Lipetsk Oblast","Russia"), ("Magadan Oblast","Russia"), ("Mari El Republic","Russia"), ("Mordovia Republic","Russia"),
        ("Moscow","Russia"), ("Moscow Oblast","Russia"), ("Murmansk Oblast","Russia"), ("Nenets Autonomous Okrug","Russia"), ("Nizhny Novgorod Oblast","Russia"),
        ("North Ossetia - Alania Republic","Russia"), ("Novgorod Oblast","Russia"), ("Novosibirsk Oblast","Russia"), ("Omsk Oblast","Russia"), ("Orel Oblast","Russia"),
        ("Orenburg Oblast","Russia"), ("Penza Oblast","Russia"), ("Perm Krai","Russia"), ("Primorsky Krai","Russia"), ("Pskov Oblast","Russia"), ("Rostov Oblast","Russia"), ("Ryazan Oblast","Russia"),
        ("Saint Petersburg","Russia"), ("Sakha (Yakutiya) Republic","Russia"), ("Sakhalin Oblast","Russia"), ("Samara Oblast","Russia"), ("Saratov Oblast","Russia"),
        ("Smolensk Oblast","Russia"), ("Stavropol Krai","Russia"), ("Sverdlovsk Oblast","Russia"), ("Tambov Oblast","Russia"), ("Tatarstan Republic","Russia"), ("Tomsk Oblast","Russia"),
        ("Tula Oblast","Russia"), ("Tver Oblast","Russia"), ("Tyumen Oblast","Russia"), ("Tyva Republic","Russia"), ("Udmurt Republic","Russia"), ("Ulyanovsk Oblast","Russia"),
        ("Vladimir Oblast","Russia"), ("Volgograd Oblast","Russia"), ("Vologda Oblast","Russia"), ("Voronezh Oblast","Russia"), ("Yamalo-Nenets Autonomous Okrug","Russia"),
        ("Yaroslavl Oblast","Russia"), ("Zabaykalsky Krai","Russia"),
    ],
    "Ukraine": [
        ("Cherkasy Oblast","Ukraine"), ("Chernihiv Oblast","Ukraine"), ("Chernivtsi Oblast","Ukraine"), ("Crimea Republic*","Ukraine"), ("Dnipropetrovsk Oblast","Ukraine"),
        ("Donetsk Oblast","Ukraine"), ("Ivano-Frankivsk Oblast","Ukraine"), ("Kharkiv Oblast","Ukraine"), ("Kherson Oblast","Ukraine"), ("Khmelnytskyi Oblast","Ukraine"),
        ("Kiev","Ukraine"), ("Kiev Oblast","Ukraine"), ("Kirovohrad Oblast","Ukraine"), ("Luhansk Oblast","Ukraine"), ("Lviv Oblast","Ukraine"), ("Mykolaiv Oblast","Ukraine"),
        ("Odessa Oblast","Ukraine"), ("Poltava Oblast","Ukraine"), ("Rivne Oblast","Ukraine"), ("Sevastopol*","Ukraine"), ("Sumy Oblast","Ukraine"), ("Ternopil Oblast","Ukraine"),
        ("Vinnytsia Oblast","Ukraine"), ("Volyn Oblast","Ukraine"), ("Zakarpattia Oblast","Ukraine"), ("Zaporizhia Oblast","Ukraine"), ("Zhytomyr Oblast","Ukraine"),
    ],
    "Sweden": [
        ("Blekinge","Sweden"), ("Dalarna","Sweden"), ("Gavleborg","Sweden"), ("Gotland","Sweden"), ("Halland","Sweden"), ("Jamtland Harjedalen","Sweden"), ("Jonkoping","Sweden"),
        ("Kalmar","Sweden"), ("Kronoberg","Sweden"), ("Norrbotten","Sweden"), ("Orebro","Sweden"), ("Ostergotland","Sweden"), ("Skane","Sweden"), ("Sormland","Sweden"),
        ("Stockholm","Sweden"), ("Uppsala","Sweden"), ("Varmland","Sweden"), ("Vasterbotten","Sweden"), ("Vasternorrland","Sweden"), ("Vastmanland","Sweden"), ("Vastra Gotaland","Sweden"),
    ],
    "India": [
        ("Andaman and Nicobar Islands","India"), ("Andhra Pradesh","India"), ("Arunachal Pradesh","India"), ("Assam","India"), ("Bihar","India"), ("Chandigarh","India"),
        ("Chhattisgarh","India"), ("Dadar Nagar Haveli","India"), ("Delhi","India"), ("Goa","India"), ("Gujarat","India"), ("Haryana","India"), ("Himachal Pradesh","India"),
        ("Jammu and Kashmir","India"), ("Jharkhand","India"), ("Karnataka","India"), ("Kerala","India"), ("Ladakh","India"), ("Madhya Pradesh","India"), ("Maharashtra","India"),
        ("Manipur","India"), ("Meghalaya","India"), ("Mizoram","India"), ("Nagaland","India"), ("Odisha","India"), ("Puducherry","India"), ("Punjab","India"), ("Rajasthan","India"),
        ("Sikkim","India"), ("Tamil Nadu","India"), ("Telangana","India"), ("Tripura","India"), ("Unknown","India"), ("Uttar Pradesh","India"), ("Uttarakhand","India"), ("West Bengal","India"),
    ],
    "Netherlands": [
    ("Aruba","Netherlands"), ("Curacao","Netherlands"), ("Drenthe","Netherlands"), ("Flevoland","Netherlands"), ("Friesland","Netherlands"), ("Gelderland","Netherlands"),
("Groningen","Netherlands"), ("Limburg","Netherlands"), ("Noord-Brabant","Netherlands"), ("Noord-Holland","Netherlands"), ("Overijssel","Netherlands"), ("Sint Maarten","Netherlands"),
("Unknown","Netherlands"), ("Utrecht","Netherlands"), ("Zeeland","Netherlands"), ("Zuid-Holland","Netherlands"),
    ("Netherlands", "Netherlands"),
    ],
}

# EU alt names
ALTERNATIVE_NAMES["EU"] = []
for c in eu_countries:
    ALTERNATIVE_NAMES["EU"].append(c)
    if c in ALTERNATIVE_NAMES:
        ALTERNATIVE_NAMES["EU"].extend(ALTERNATIVE_NAMES[c])

# US alt names
us_countries_alt_names = [(c,"US") for c in us_countries]
ALTERNATIVE_NAMES["USA"] = us_countries_alt_names
for c in us_countries:
    ALTERNATIVE_NAMES[c] = [(c,"US")]

# Canada alt names
for c in canada_countries:
    ALTERNATIVE_NAMES[c] = [(c,"Canada")]
ALTERNATIVE_NAMES["Ontario"].extend([("Toronto, ON", "Canada"), ("London, ON", "Canada")])
ALTERNATIVE_NAMES["Canada"] = [(c,"Canada") for c in canada_countries]
ALTERNATIVE_NAMES["Canada"].extend(ALTERNATIVE_NAMES["Ontario"])

# JHU data for US from 2/1 to 3/9 are little bit strange, so we need to add following
AZ_EXTRA_NAMES = ["Maricopa County, AZ", "Pinal County, AZ", "Tempe, AZ"]
CA_EXTRA_NAMES = ["Alameda County, CA", "Berkeley, CA", "Contra Costa County, CA", "Fresno County, CA", "Humboldt County, CA",
    "Los Angeles, CA", "Madera County, CA", "Orange, CA", "Orange County, CA", "Placer County, CA", "Riverside County, CA",
    "Sacramento County, CA", "San Benito, CA", "San Diego County, CA", "San Francisco County, CA", "San Mateo, CA", "Santa Clara, CA",
    "Santa Clara County, CA", "Santa Cruz County, CA", "Shasta County, CA", "Sonoma County, CA", "Travis, CA", "Yolo County, CA"
]
CO_EXTRA_NAMES = ["Denver County, CO", "Douglas County, CO", "El Paso County, CO", "Summit County, CO"]
CT_EXTRA_NAMES = ["Fairfield County, CT"]
DC_EXTRA_NAMES = ["Washington, D.C."]
FL_EXTRA_NAMES = ["Broward County, FL", "Charlotte County, FL", "Hillsborough, FL", "Lee County, FL", "Manatee County, FL",
    "Okaloosa County, FL", "Santa Rosa County, FL", "Sarasota, FL", "Volusia County, FL"]
GA_EXTRA_NAMES = ["Cherokee County, GA", "Cobb County, GA", "Floyd County, GA", "Fulton County, GA", "Polk County, GA"]
IA_EXTRA_NAMES = ["Johnson County, IA"]
IL_EXTRA_NAMES = ["Chicago, IL", "Cook County, IL"]
IN_EXTRA_NAMES = ["Hendricks County, IN", "Marion County, IN"]
KS_EXTRA_NAMES = ["Johnson County, KS"]
KY_EXTRA_NAMES = ["Fayette County, KY", "Harrison County, KY", "Jefferson County, KY"]
LA_EXTRA_NAMES = ["Jefferson Parish, LA"]
MA_EXTRA_NAMES = ["Berkshire County, MA", "Boston, MA", "Middlesex County, MA", " Norfolk County, MA", "Norfolk County, MA",
    "Norwell County, MA", "Plymouth County, MA", "Suffolk County, MA", "Unknown Location, MA"]
MD_EXTRA_NAMES = ["Harford County, MD", "Montgomery County, MD"]
MN_EXTRA_NAMES = ["Carver County, MN", "Ramsey County, MN"]
MO_EXTRA_NAMES = ["St. Louis County, MO"]
NC_EXTRA_NAMES = ["Chatham County, NC", "Wake County, NC"]
NE_EXTRA_NAMES = ["Ashland, NE", "Douglas County, NE"]
NH_EXTRA_NAMES = ["Grafton County, NH", "Rockingham County, NH"]
NJ_EXTRA_NAMES = ["Bergen County, NJ", "Hudson County, NJ"]
NV_EXTRA_NAMES = ["Clark County, NV", "Washoe County, NV"]
NY_EXTRA_NAMES = ["Nassau County, NY", "New York City, NY", "New York County, NY", "Queens County, NY", "Rockland County, NY",
    "Saratoga County, NY", "Suffolk County, NY", "Ulster County, NY", "Westchester County, NY"]
OK_EXTRA_NAMES = ["Tulsa County, OK"]
OR_EXTRA_NAMES = ["Jackson County, OR ", "Douglas County, OR", "Klamath County, OR", "Marion County, OR", "Portland, OR",
    "Umatilla, OR", "Washington County, OR"]
PA_EXTRA_NAMES = ["Delaware County, PA", "Montgomery County, PA", "Wayne County, PA"]
RI_EXTRA_NAMES = ["Providence County, RI", "Providence, RI"]
SC_EXTRA_NAMES = ["Charleston County, SC", "Kershaw County, SC", "Spartanburg County, SC"]
TN_EXTRA_NAMES = ["Davidson County, TN", "Shelby County, TN", "Williamson County, TN"]
TX_EXTRA_NAMES = [ "Collin County, TX", "Fort Bend County, TX", "Harris County, TX", "Lackland, TX", "Montgomery County, TX", "San Antonio, TX"]
UT_EXTRA_NAMES = ["Davis County, UT"]
VA_EXTRA_NAMES = ["Fairfax County, VA"]
VT_EXTRA_NAMES = ["Bennington County, VT"]
WA_EXTRA_NAMES = ["Clark County, WA", "Grant County, WA", "Jefferson County, WA", "King County, WA", "Kittitas County, WA", "Pierce County, WA",
    "Seattle, WA", "Snohomish County, WA", "Spokane County, WA"]
WI_EXTRA_NAMES = ["Madison, WI"]

ALTERNATIVE_NAMES["California"].extend((n, "US") for n in CA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Virginia"].extend((n, "US") for n in VA_EXTRA_NAMES)
ALTERNATIVE_NAMES["New York"].extend((n, "US") for n in NY_EXTRA_NAMES)
ALTERNATIVE_NAMES["Arizona"].extend((n, "US") for n in AZ_EXTRA_NAMES)
ALTERNATIVE_NAMES["Colorado"].extend((n, "US") for n in CO_EXTRA_NAMES)
ALTERNATIVE_NAMES["Connecticut"].extend((n, "US") for n in CT_EXTRA_NAMES)
ALTERNATIVE_NAMES["Florida"].extend((n, "US") for n in FL_EXTRA_NAMES)
ALTERNATIVE_NAMES["Georgia"].extend((n, "US") for n in GA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Iowa"].extend((n, "US") for n in IA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Illinois"].extend((n, "US") for n in IL_EXTRA_NAMES)
ALTERNATIVE_NAMES["Indiana"].extend((n, "US") for n in IN_EXTRA_NAMES)
ALTERNATIVE_NAMES["Kansas"].extend((n, "US") for n in KS_EXTRA_NAMES)
ALTERNATIVE_NAMES["Kentucky"].extend((n, "US") for n in KY_EXTRA_NAMES)
ALTERNATIVE_NAMES["Louisiana"].extend((n, "US") for n in LA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Massachusetts"].extend((n, "US") for n in MA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Maryland"].extend((n, "US") for n in MD_EXTRA_NAMES)
ALTERNATIVE_NAMES["Minnesota"].extend((n, "US") for n in MN_EXTRA_NAMES)
ALTERNATIVE_NAMES["Missouri"].extend((n, "US") for n in MO_EXTRA_NAMES)
ALTERNATIVE_NAMES["North Carolina"].extend((n, "US") for n in NC_EXTRA_NAMES)
ALTERNATIVE_NAMES["Nebraska"].extend((n, "US") for n in NE_EXTRA_NAMES)
ALTERNATIVE_NAMES["New Hampshire"].extend((n, "US") for n in NH_EXTRA_NAMES)
ALTERNATIVE_NAMES["New Jersey"].extend((n, "US") for n in NJ_EXTRA_NAMES)
ALTERNATIVE_NAMES["Nevada"].extend((n, "US") for n in NV_EXTRA_NAMES)
ALTERNATIVE_NAMES["New York"].extend((n, "US") for n in NY_EXTRA_NAMES)
ALTERNATIVE_NAMES["Oklahoma"].extend((n, "US") for n in OK_EXTRA_NAMES)
ALTERNATIVE_NAMES["Oregon"].extend((n, "US") for n in OR_EXTRA_NAMES)
ALTERNATIVE_NAMES["Pennsylvania"].extend((n, "US") for n in PA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Rhode Island"].extend((n, "US") for n in RI_EXTRA_NAMES)
ALTERNATIVE_NAMES["South Carolina"].extend((n, "US") for n in SC_EXTRA_NAMES)
ALTERNATIVE_NAMES["Tennessee"].extend((n, "US") for n in TN_EXTRA_NAMES)
ALTERNATIVE_NAMES["Texas"].extend((n, "US") for n in TX_EXTRA_NAMES)
ALTERNATIVE_NAMES["Utah"].extend((n, "US") for n in UT_EXTRA_NAMES)
ALTERNATIVE_NAMES["Vermont"].extend((n, "US") for n in VT_EXTRA_NAMES)
ALTERNATIVE_NAMES["Washington"].extend((n, "US") for n in WA_EXTRA_NAMES)
ALTERNATIVE_NAMES["Wisconsin"].extend((n, "US") for n in WI_EXTRA_NAMES)

US_EXTRA_NAMES = AZ_EXTRA_NAMES + CA_EXTRA_NAMES + CO_EXTRA_NAMES + CT_EXTRA_NAMES + DC_EXTRA_NAMES + FL_EXTRA_NAMES + GA_EXTRA_NAMES + IA_EXTRA_NAMES +\
        IL_EXTRA_NAMES + IN_EXTRA_NAMES + KS_EXTRA_NAMES + KY_EXTRA_NAMES + LA_EXTRA_NAMES + MA_EXTRA_NAMES + MD_EXTRA_NAMES + MN_EXTRA_NAMES +\
        MO_EXTRA_NAMES + NC_EXTRA_NAMES + NE_EXTRA_NAMES + NH_EXTRA_NAMES + NJ_EXTRA_NAMES + NV_EXTRA_NAMES + NY_EXTRA_NAMES + OK_EXTRA_NAMES +\
        OR_EXTRA_NAMES + PA_EXTRA_NAMES + RI_EXTRA_NAMES + SC_EXTRA_NAMES + TN_EXTRA_NAMES + TX_EXTRA_NAMES + UT_EXTRA_NAMES + VA_EXTRA_NAMES +\
        VT_EXTRA_NAMES + WA_EXTRA_NAMES + WI_EXTRA_NAMES
ALTERNATIVE_NAMES["USA"].extend((n, "US") for n in US_EXTRA_NAMES)
##### ^^ JHU US DATA fuckery

TEMPERATURES = {
    "Czechia": "Prague_LKPR",
    "Alberta": "Canada_Alberta",
    "British Columbia": "Canada_British_Columbia",
    "Nova Scotia": "Canada_Nova_Scotia",
    "Canada Ontario": "Canada_Ontario",
    "Canada Quebec": "Canada_Quebec",
    "Serbia": "Serbia",
    "Ukraine": "Ukraine",
    "Switzerland": "Switzerland",
    "Moldova": "Moldova",
    "Iceland": "Iceland",
    "Singapore": "Singapore",
    "Taiwan": "Taiwan",
    "New Zealand": "New_Zealand",
    "South Korea": "South_Korea",
    "Vietnam": "Vietnam",
    "Seychelles": "Seychelles",
    "South Africa": "South Africa",
    "Uruguay": "Uruguay",
    "Kazakhstan": "Kazakhstan",
    "Saudi Arabia": "Saudi Arabia",
#    "Israel": "Israel",
    "Netherlands": "Netherlands",
    "Italy": "Italy",
    "Ireland": "Ireland",
    "Denmark": "Denmark",
    "Spain": "Spain",
    "Luxembourg": "Luxembourg",
    "Slovenia": "Slovenia",
    "Greece": "Greece",
    "Sweden": "Sweden",
    "Malta": "Malta",
    "Alabama": "Alabama",
    "Alaska": "Alaska",
    "California": "California",
    "Colorado": "Colorado",
    "New York": "New York",
    "Texas": "Texas",
    "Virginia": "Virginia",
    "Florida": "Florida",
    "Washington": "Washington",
}

POPULATION = {
    "EU": 445000000,
    "Singapore": 5700000,
    "Czechia": 10700000,
    "Netherlands":  17100000,
    "Italy": 60500000,
    "Taiwan": 23800000,
    "New Zealand": 5000000,
    "Slovakia": 5500000,
    "Japan": 126000000,
    "South Korea": 51000000,
    "Ireland": 5000000,
    "Denmark": 5800000,
    "Serbia": 8500000,
    "Vietnam": 97000000,
    "Albania": 2800000,
    "Spain": 46700000,
    "Seychelles": 100000,
    "Luxembourg": 600000,
    "Slovenia": 2000000,
    "Greece": 10500000,
    "Guam (US)": 170000,
    "Austria": 8800000,
    "Belgium": 11300000,
    "Bulgaria": 7000000,
    "Croatia": 4100000,
    "Cyprus": 854800,
    "Estonia": 1300000,
    "Finland": 5600000,
    "France": 67000000,
    "Germany": 82000000,
    "Hungary": 9700000,
    "Latvia": 1900000,
    "Lithuania": 2800000,
    "Malta": 440000,
    "Poland": 38000000,
    "Portugal": 10300000,
    "Romania": 19600000,
    "Sweden": 10000000,
    "UK": 67800000,
    "Russia": 145000000,
    "USA": 325000000,
    "Algeria": 43000000,
    # US
    "Alabama": 4900000,
    "Alaska": 730000,
    "Arizona": 7300000,
    "Arkansas": 3000000,
    "California": 40000000,
    "Colorado": 5750000,
    "Connecticut": 3500000,
    "Delaware": 970000,
    "Florida": 21500000,
    "Georgia": 10600000,
    "Hawaii": 1400000,
    "Idaho": 1787000,
    "Illinois": 12670000,
    "Indiana": 6700000,
    "Iowa": 3100000,
    "Kansas": 2900000,
    "Kentucky": 4460000,
    "Louisiana": 4460000,
    "Maine": 1300200,
    "Maryland": 6000000,
    "Massachusetts": 6900000,
    "Michigan": 9900000,
    "Minnesota": 5600000,
    "Mississippi": 3000000,
    "Missouri": 6100000,
    "Montana": 1000000,
    "Nebraska": 1900000,
    "Nevada": 3000000,
    "New Hampshire": 1300000,
    "New Jersey": 8800000,
    "New Mexico": 2000000,
    "New York": 19500000,
    "North Carolina": 10500000,
    "North Dakota": 760000,
    "Ohio": 11700000,
    "Oklahoma": 3950000,
    "Oregon": 4200000,
    "Pennsylvania": 12800000,
    "Rhode Island": 1000000,
    "South Carolina": 5100000,
    "South Dakota": 850000,
    "Tennessee": 6800000,
    "Texas": 29000000,
    "Utah": 3200000,
    "Vermont": 620000,
    "Virginia": 8600000,
    "Washington": 7600000,
    "West Virginia": 1700000,
    "Wisconsin": 5800000,
    "Wyoming": 567000,
    # Canada
    "Canada": 37000000,
    "Alberta": 4000000,
    "British Columbia": 4600000,
    "Manitoba": 1250000,
    "New Brunswick": 750000,
    "Newfoundland and Labrador": 520000,
    "Northwest Territories": 40000,
    "Nova Scotia": 920000,
    "Ontario": 13400000,
    "Prince Edward Island": 140000,
    "Quebec": 8160000,
    "Saskatchewan": 1000000,
    "Yukon": 35000,
    # Others
    "Ukraine": 41000000,
    "Kazakhstan": 18000000,
    "Uzbekistan": 33000000,
    "Kyrgyzstan": 6500000,
    "Mongolia": 3000000,
    "Iraq": 40000000,
    "Iran": 83000000,
    "Saudi Arabia": 33000000,
    "Jordan": 10000000,
    "Israel": 9000000,
    "Turkey": 84000000,
    "Egypt": 100000000,
    "Senegal": 16500000,
    "Burkina Faso": 20000000,
    "Nigeria": 200000000,
    "Morocco": 36500000,
    "Belarus": 9500000,
    "Switzerland": 8600000,
    "Moldova": 4000000,
    "Bosnia and Herzegovina": 3200000,
    "India":  1377000000,
    "Pakistan": 220000000,
    "Nepal": 29000000,
    "Malaysia": 32000000,
    "Thailand": 70000000,
    "Laos": 7200000,
    "Philippines": 109000000,
    "Indonesia": 270000000,
    "Papua New Guinea": 9000000,
    "Australia": 25700000,
    "Iceland": 340800,
    "Mexico": 128000000,
    "Cuba": 11000000,
    "Guatemala": 17000000,
    "Venezuela": 28000000,
    "Costa Rica": 5000000,
    "Peru": 32800000,
    "Brazil": 212000000,
    "Bolivia": 11500000,
    "Chile": 19000000,
    "Argentina": 45000000,
    "Uruguay": 3500000,
    "Colombia": 50800000,
    "Paraguay": 7000000,
    "Faroe Islands": 50000,
    "Falkland Islands": 3500,
    "Greenland": 57000,
    "Tunisia": 10800000,
    "Mali": 20000000,
    "Kenya": 53000000,
    "Ethipoia": 115000000,
    "Isle of Man": 86000,
    "South Africa": 58800000,
    "Abruzzo (Italy region)": 1307309,
    "Basilicata (Italy region)": 578036,
    "P.A. Bolzano (Italy region)":  520891,
    "Calabria (Italy region)": 1959050,
    "Campania (Italy region)": 5766810,
    "Emilia-Romagna (Italy region)": 4342135,
    "Friuli Venezia Giulia (Italy region)": 1220291,
    "Lazio (Italy region)": 5502886,
    "Liguria (Italy region)": 1570694,
    "Lombardia (Italy region)": 9704151,
    "Marche (Italy region)": 1541319,
    "Molise (Italy region)": 313660,
    "Piemonte (Italy region)": 4363916,
    "Puglia (Italy region)": 4052566,
    "Sardegna (Italy region)": 1639362,
    "Sicilia (Italy region)": 5002904,
    "Toscana (Italy region)": 3529946,
    "P.A. Trento (Italy region)": 1029475,
    "Umbria (Italy region)": 884268,
    "Valle d'Aosta (Italy region)": 126806,
    "Veneto (Italy region)": 4855904,
    "Cambodia": 17000000,
    # regions
    "Chiapas (Mexico)": 5500000,
}

# annotation last update: 2020-06-05
# (previous update: 2020-05-07)
EVENTS = {
    # country: [(YYYY-MM-DD, label, color, line-override-dict, font-override-dict, annotation-override-dict)]
    "Czechia": [
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_the_Czech_Republic
        ("2020-03-11", "Schools closed", "black", None, None, {'y': 0.7}),
        ("2020-03-12", "State of emergency", "red", None, None, {'y': 0.7, 'xshift': -5}),
        ("2020-03-14", "Borders closed", "red", None, None, {'y': 0.93}),
        ("2020-03-16", "Stay-at-home", "red", None, None, {'y': 0.93}),
        ("2020-03-19", "Face masks compulsory", "blue", None, None, {'y': 0.93}),

        # https://www.novinky.cz/koronavirus/clanek/zapouzdrene-cesko-velke-srovnani-otevirani-obchodu-a-sluzeb-se-zahranicim-40320828
        # https://koronavirus.mzcr.cz/uvolnovani-opatreni/
        ("2020-04-09", "Non-essential shops opened", "black", {"dash":"dot"}, None, {'y': 0.93}),
        ("2020-04-20", "More shops opened", "black", {"dash":"dot"}, None),
        ("2020-04-27", "More shops, gyms and partially borders opened", "red", {"dash":"dot"}, None),
        ("2020-05-11", "Shopping malls, almost everything opened", "black", {"dash":"dot"}, None),
        ("2020-05-25", "bars, restaurants, borders opened ...", "red", {"dash":"dot"}, None),
#        ("2020-06-08", "gatherings upto 500 people allowed", "black", {"dash":"dot"}, None),
        ("2020-10-28", "lockdown", "red", None, None, {'y': 0.93}),
#       https://www.vlada.cz/cz/epidemie-koronaviru/dulezite-informace/nouzovy-stav-a-mimoradna-opatreni-_-co-aktualne-plati-180234/
        ("2021-03-01", "total lockdown", "red", None, None, {'y': 0.93}),
    ],
    "Slovakia": [
        ("2020-03-09", "Some schools closed", "black", None, None, {'y': 0.95}),
        ("2020-03-12", "State of emergency", "red", None, None, {'y': 0.95}),
        ("2020-03-13", "Borders and airports closed, stay at home", "red", None, None, {'y': 0.95}),
        ("2020-03-14", "Face masks compulsory - public transport, shops", "blue", None, None, {'y': 0.95}),

        # https://www.health.gov.sk/Clanok?Hlavna-sprava-COVID-19 (archived - archive.is)
        ("2020-04-22", "Non-essential shops opened", "black", {"dash":"dot"}, None, {'y':0.9}),
        ("2020-05-05", "Shops and some services opened", "black", {"dash":"dot"}, None, {'y':0.9}),
        ("2020-05-20", "More shops, shopping malls opened", "black", {"dash":"dot"}, None, {'y':0.9}),
        ("2020-05-27", "Borders partially opened", "red", {"dash":"dot"}, None, {'y':0.9}),
        ("2020-06-01", "Schools opened", "black", {"dash":"dot"}, None, {'y':0.9}),
        ("2020-06-03", "Indoor sports activities allowed", "black", {"dash":"dot"}, None, {'y':0.9}),
        # http://archive.is/Yg6Wx
        # Hranice - zmena od 5. júna 2020 od 8. hodiny
        # Bez povinnosti karantény alebo predloženia negatívneho testu na COVID-19 môžu bez časového ohraničenia
        # prísť na Slovensko všetky osoby s trvalým alebo prechodným pobytom alebo osoby zdržiavajúce sa na území Českej republiky,
        # Maďarska, Rakúskej republiky alebo na území Slovenskej republiky, ktoré vstupujú do SR cez hraničné priechody z ČR,
        # Maďarska a Rakúskej republiky. 
        ("2020-06-05", "Borders opened", "red", {"dash":"dot"}, None, {'y':0.9}),
    ],
    "New Zealand": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_New_Zealand
        # https://www.beehive.govt.nz/release/major-steps-taken-protect-new-zealanders-covid-19#_edn1
        ("2020-03-14", "Everyone entering NZ to self-isolate", "black"),
        ("2020-03-20", "Borders closed (non-residents)", "red", None, None, {'y': 0.8}),
        ("2020-03-26", "Full lockdown (level 4)", "red"),
        # https://www.beehive.govt.nz/release/pm-announces-date-move-alert-level-3
        ("2020-04-28", "Lockdown restrictions eased (level 3)", "red", {"dash":"dot"}),
        # https://www.beehive.govt.nz/release/new-zealand-moves-out-state-national-emergency
        ("2020-05-13", "National emergency lifted", "red", {"dash":"dot"}),
        # https://www.health.govt.nz/our-work/diseases-and-conditions/covid-19-novel-coronavirus
        # http://archive.is/mswJd
        ("2020-05-14", "Alert level 2", "black", {"dash":"dot"}, None, {'xshift': 20}),
        # https://www.nzherald.co.nz/nz/news/article.cfm?c_id=1&objectid=12336288
        ("2020-06-01", "Huge gatherings (protests)", "red", {"dash":"dot"}),
        # https://www.dailymail.co.uk/news/article-8397579/New-Zealand-ERADICATES-coronavirus-COVID-19-Jacinda-Ardern-says-ZERO-active-cases.html
        # https://www.itv.com/news/2020-06-16/new-zealand-records-two-new-cases-of-coronavirus/
    ],
    "Taiwan": [
        # https://www.taiwannews.com.tw/en/news/3925143
        ("2020-02-02", "Schools temporary closed (to Feb 25)", "black"),
        ("2020-03-13", "Entering persons to self-isolate", "black", None, None, {'y': 0.95}),
        ("2020-03-19", "Borders closed (non-residents)", "red", None, {"color": "black"}, {'y': 0.95}),
    ],
    "Singapore": [
        # https://www.straitstimes.com/askst/wuhan-virus-who-needs-to-wear-a-mask-and-whats-the-proper-way-to-wear-it
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Singapore#April_2020_2
        ("2020-01-31", "Citizens DISCOURAGED from wearing masks", "blue", {"dash":"dot"}),

        ("2020-03-02", "Banned vistors from IR, IT and KO", "black", None, None, {'y': 0.5}),
        ("2020-03-13", "Banned vistors from FR, DE, ES", "black", None, None, {'y': 0.95}),
        ("2020-03-18", "Entering persons to self-isolate", "black"),
        ("2020-03-20", "Bluetooth tracing launched (TraceTogether)", "black"),
        ("2020-03-23", "All short-term visitors banned", "red"),
        ("2020-03-28", "Stay-at-home advice", "black"),

        ("2020-04-07", "Stay-at-home", "red"),
        ("2020-04-14", "Face masks compulsory", "blue"),

        # https://www.gov.sg/article/ending-circuit-breaker-phased-approach-to-resuming-activities-safely
        ("2020-06-02", "Re-opening: phase1 - schools and shops", "red", {"dash":"dot"}),
    ],
    "Italy": [
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Italy#Nationwide_measures
        ("2020-02-22", "11 municipalities quarantined", "black"),
        ("2020-03-01", "Quarantine extended, regional partial lockdown", "black", None, None, {'y': 0.6}),
        ("2020-03-04", "Schools closed nationwide", "black", None, None, {'y': 0.95}),
        ("2020-03-09", "Nationwide partial lockdown", "red"),
        ("2020-03-21", "Nationwide lockdown", "red", None, {"color": "black"}, {'y': 0.75}),
        # https://www.bbc.com/news/amp/world-europe-52435273
        ("2020-05-04", "Lockdown eased, restaurants takeaway allowed", "black", {"dash":"dot"}, {"color": "black"}, {'y': 0.75}),
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Italy#Nationwide_measures
        ("2020-05-16", "Reopening - most shops", "red", {"dash":"dot"}),
        ("2020-05-25", "Swimming pools and gym opened", "red", {"dash":"dot"}),
    ],
    "Netherlands": [
        ("2020-03-10", "Social distancing adviced in one region", "black", None, None, {'y':0.6}),
        ("2020-03-12", "Events with >100 people banned", "black", None, None, {'y':0.6}),
        ("2020-03-15", "Schools closed", "black"),
        ("2020-03-23", "Strict social distancing", "black"),

        # https://www.government.nl/topics/coronavirus-covid-19/news/2020/05/06/avoid-busy-places-and-stay-1.5-metres-away-from-others
        ("2020-05-11", "Restrictions eased. Primary schools reopen.", "black", {"dash":"dot"}, None),
        # https://www.government.nl/topics/coronavirus-covid-19/tackling-new-coronavirus-in-the-netherlands/changes-to-measures-from-11-may
        ("2020-06-01", "Shops, restaurants ... reopen", "black", {"dash":"dot"}, None),
    ],
    "Japan": [
        ("2020-02-05", "Diamond Princess quarantined", "white", None, {"color": "black"}, None),
        ("2020-02-27", "Schools closed", "black"),
        ("2020-03-30", "Stay-at-home advice in Tokyo", "black"),
        ("2020-04-07", "State of emergency - regional", "red"),
        ("2020-04-10", "Big venues closed in Tokyo", "black"),
        ("2020-04-16", "State of emergency", "red"),
        # https://www.bbc.com/news/world-asia-52658551
        # https://www.japantimes.co.jp/news/2020/05/14/national/japan-coronavirus-emergency-39-prefectures/
        ("2020-05-14", "State of emegency lifted in most regions", "red", {"dash":"dot"}, None),
    ],
    "South Korea": [
        # https://www.bbc.com/news/world-asia-51582186
        # https://www.koreatimes.co.kr/www/nation/2020/02/119_283825.html
        ("2020-02-21", "Daegu and Cheongdo - special care zones", "black"),
        ("2020-03-19", "All entering persons to self-isolate", "black"),
        ("2020-04-13", "Borders closed (non-residents)", "black"),
    ],
    "Ireland": [
        # https://www.gov.ie/en/publication/ce3fe8-previous-updates-on-covid-19-coronavirus/
        ("2020-03-09", "St Patrick's day cancelled", "black"),
        ("2020-03-13", "Schools closed", "black"),
        ("2020-03-15", "Pubs advised to close", "black"),
        ("2020-03-20", "State of emergency", "red"),
        ("2020-03-27", "Stay-at-home", "red"),

        # https://www.gov.ie/en/news/58bc8b-taoiseach-announces-roadmap-for-reopening-society-and-business-and-u/
        ("2020-05-18", "Reopening - phase 1", "red", {"dash":"dot"}, None),

        # https://www.gov.ie/en/campaigns/c36c85-covid-19-coronavirus/
        # https://www.gov.ie/en/news/58bc8b-taoiseach-announces-roadmap-for-reopening-society-and-business-and-u/
        # The guidance to stay at home has been extended until Monday 18 May because the measures are working.
        # By staying at home - you are saving lives.
    ],
    "Denmark": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Denmark
        ("2020-03-11", "advice to cancel events with >100 persons", "black"),
        ("2020-03-13", "stay-at-home", "red"),
        ("2020-03-14", "borders closed", "red"),
        ("2020-03-18", "shopping centers, other shops closed", "red"),
        # https://politi.dk/en/coronavirus-in-denmark/controlled-reopening-of-the-danish-society
        ("2020-04-15", "some schools opened", "black", {"dash": "dot"}),
        ("2020-04-20", "more shops opened", "black", {"dash": "dot"}),
        
        # https://politi.dk/en/coronavirus-in-denmark/extension-of-measures-during-the-covid19-outbreak-in-denmark
        # The measures are in force until at least 11 May 2020.

        # https://politi.dk/en/coronavirus-in-denmark/controlled-reopening-of-denmark
        ("2020-05-11", "full reopening of retail", "red", {"dash": "dot"}),
        ("2020-05-18", "easing more restrictions (cafes, schools)", "black", {"dash": "dot"}),
        ("2020-05-27", "easing more restrictions", "black", {"dash": "dot"}),
    ],
    "Serbia": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Serbia
        ("2020-03-15", "state of emergency, schools closed, borders closed", "red", None, None, {'y': 0.7}),
        ("2020-03-17", "night curfew, people over 65 total curfew", "red", None, None, {'y': 0.6}),
        ("2020-03-21", "public transport and non-essential shops closed", "black", None, None, {'y': 0.7}),
        
        # https://rs.usembassy.gov/
        # https://covid19.rs/homepage-english/
        # http://www.zslaw.rs/serbia-lifts-state-of-emergency-and-cancels-curfew/
        # https://rs.usembassy.gov/health-and-travel-alert-may-8-2020/
        ("2020-05-07", "State of emergency and curfew cancelled", "red", {"dash":"dot"}, None, {'y':0.9}),
        # https://rs.usembassy.gov/health-and-travel-alert-may-22-2020/
        ("2020-05-22", "All entry restrictions lifted", "red", {"dash":"dot"}, None, {'y':0.9}),
    ],
    "Vietnam": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Vietnam
        # https://e.vnexpress.net/news/news/saigon-mosque-closed-after-followers-contact-with-covid-19-patient-4071122.html
        # https://vn.usembassy.gov/u-s-citizen-services/covid-19-information/
        ("2020-03-17", "Saigon mosque closed ", "black", None, None),
        ("2020-03-22", "borders closed for foreigners", "red", None, None),
        # http://news.chinhphu.vn/Home/Viet-Nam-to-go-into-15day-nationwide-social-distancing-to-curb-COVID19/20203/39472.vgp
        ("2020-04-01", "stay-at-home advised", "black", None, None),
        # http://news.chinhphu.vn/Home/Ha-Noi-lifts-lockdown-on-COVID19-hotspot/20205/39998.vgp
        ("2020-04-08", "Me Linh lockdown", "red", None, None),
        # 2020/04/15  stay-at-home extended
        # http://news.chinhphu.vn/Home/Govt-extends-social-distancing-for-at-least-one-week-in-28-localities/20204/39735.vgp
        # 2020/04/22 <- further extensions should be decided?
        # 2020/04/15  stay-at-home extended
        # http://news.chinhphu.vn/Home/Ha-Noi-lifts-lockdown-on-COVID19-hotspot/20205/39998.vgp
        ("2020-05-06", "Me Linh lockdown lifted", "red", {"dash":"dot"}, None),
    ],
    "Albania": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Albania
        # ("2020-03-08", "Fligths and ferries from Italy cancelled, schools closed", "black", None, None),
        ("2020-03-10", "Partial lockdown", "red", None, None),
        ("2020-03-12", "Lockdown and curfew", "red", None, None),
        # https://www.garda.com/crisis24/news-alerts/323096/albania-borders-closed-as-of-march-16-due-to-covid-19-update-2
        ("2020-03-15", "Borders closed", "red", None, None),
        # https://www.albaniandailynews.com/index.php?idm=41526&mod=2
        ("2020-04-20", "Some shops opened", "black", None, None),
        ("2020-04-26", "Curfew eased slightly", "black", {"dash":"dot"}, None),
        # https://www.albaniandailynews.com/
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Albania#Relaxation_of_measures
        # https://www.albaniandailynews.com/index.php?idm=42960&mod=2
        ("2020-05-18", "Restrictions eased", "red", {"dash":"dot"}, None),
        ("2020-06-01", "Furter restrictions eased", "black", {"dash":"dot"}, None),
    ],
    "Spain": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Spain
        ("2020-03-07", "One region quarantine (Haro)", "black", None, None),
        ("2020-03-12", "Further regional quarantines", "black", None, None, {'y': 0.95}),
        ("2020-03-15", "state of alarm", "red", None, None),
        ("2020-03-16", "borders closed for foreigners", "red", None, {"size": 10}, {'xshift': -2}),
        ("2020-03-28", "stay at home", "red", None, None, {'y': 0.95}),
        ("2020-04-13", "nonessential workers allowed to work", "black", {"dash":"dot"}, None),
        # https://www.thelocal.es/20200428/what-we-know-about-spains-plan-for-transition-to-a-new-normal
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Spain#De-escalation_(2_May%E2%80%93)
        ("2020-05-02", "lockdown eased slightly", "black", {"dash":"dot"}, None),
        # https://www.thelocal.es/20200508/what-you-can-and-cant-do-during-phase-1
        ("2020-05-11", "further restrictions eased", "black", {"dash":"dot"}, None),
        # https://www.thelocal.es/20200522/these-are-the-provinces-advancing-to-phase-1-and-phase-2
        ("2020-05-25", "further restrictions eased", "black", {"dash":"dot"}, None),
        # https://www.thelocal.es/20200605/madrid-and-barcelona-advance-to-phase-2-here-are-the-new-rules
        # ("2020-06-08", "phase3 opening (madrid, barcelona phase2)", "red", {"dash":"dot"}, None),
    ],
    "Seychelles": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Seychelles
        # http://www.seychellesnewsagency.com/search_news?do_search=no&thequery=covid-19&&s=0 (news)
        # http://www.seychellesnewsagency.com/articles/12688/Seychelles+and+COVID-++measures+taken+by+the+island+nation+to+protect+its+people
#        ("2020-03-09", "closed for cruise ships", "black", None, None),

        ("2020-03-16", "public schools closed", "black", None, None),
        ("2020-03-18", "banned all europeans", "black", None, None),
        # http://www.seychellesnewsagency.com/articles/12626/Seychelles+and+COVID-+New+major+measures+travel+banned+for++days%2C+schools+and+daycares+closed%2C+salaries+assured
        ("2020-03-20", "schools closed", "black", None, None),
        ("2020-03-23", "banned all visitors", "red", None, None),
        # http://www.seychellesnewsagency.com/articles/12665/Seychelles+and+COVID-+Shops+to+close+at+.+pm%3B+-person+limit+on+public+gatherings
        ("2020-03-27", "banned >4 people gatherings", "black", None, None),
        # http://www.seychellesnewsagency.com/articles/12847/A+partial+re-opening+Seychelles+takes+first+small+step+toward+return+to+normal
        ("2020-05-04", "some shops and services opened", "black", {"dash":"dot"}, None),
        ("2020-05-11", "daycare opened", "black", {"dash":"dot"}, None),
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Seychelles#Lifting_of_Restrictions
        # http://www.seychellesnewsagency.com/articles/12813/Seychelles+and+COVID-+Movement+restrictions+to+be+lifted+next+week%3B+schools%2C+daycare+to+reopen+later+in+May
        ("2020-05-18", "schools opened", "black", {"dash":"dot"}, None),
        ("2020-06-01", "international aiport re-opened", "red", {"dash":"dot"}, None),
    ],
    "Luxembourg": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Luxembourg
        # https://today.rtl.lu/news/luxembourg/a/1483941.html
        ("2020-03-16", "schools closed, partial stay at home", "black", None, None, {'y':0.78}),
        ("2020-03-19", "state of emergency", "red", None, None),
        # https://today.rtl.lu/news/luxembourg/a/1502064.html
        ("2020-04-20", "some shops opened, face masks mandatory", "black", {"dash":"dot"}, None),
        # https://gouvernement.lu/en/dossiers.gouv_msan%2Ben%2Bdossiers%2B2020%2Bcorona-virus.html
        # https://today.rtl.lu/news/luxembourg/a/1514341.html
        # https://today.rtl.lu/news/luxembourg/a/1518275.html
       ("2020-05-11", "restrictions eased, schools opened", "red", {"dash":"dot"}, None),
    ],
    "Slovenia": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Slovenia
        # https://www.gov.si/en/topics/coronavirus-disease-covid-19/
        # https://www.gov.si/en/news/2020-03-12-slovenia-to-declare-an-epidemic-and-temporarily-close-kindergartens-and-schools/
        # https://www.gov.si/en/topics/coronavirus-disease-covid-19/measures-to-curb-the-spread-of-the-coronavirus/
        ("2020-03-10", "flights from CN, KO, IR, IT cancelled", "black", None, None),
        ("2020-03-12", "'epidemic declared'", "red", None, None),
        ("2020-03-16", "schools closed, partial stay at home", "black", None, None),
        # https://www.total-slovenia-news.com/politics/5895-covid-19-slovenia
        ("2020-03-20", "partial lockdown", "red", None, None),
        ("2020-03-30", "lockdown more strict", "black", None, None),
        # https://www.neweurope.eu/article/slovenia-to-ease-coronavirus-lockdown-from-april-20/
        # https://www.total-slovenia-news.com/lifestyle/6091-less-demand-than-expected-as-some-stores-reopen-in-slovenia-push-to-open-bars-restaurants-15-may
        # https://www.gov.si/en/news/2020-04-14-crossing-the-state-border-during-the-coronavirus-epidemic-40860/
        ("2020-04-11", "strict border checks, entering 7-day quarantine", "red", None, {'color': 'black'}),
        ("2020-04-18", "some sports activities permited", "black", {"dash": "dot"}, None),
        ("2020-04-20", "non-essential shops opened", "black", {"dash": "dot"}, None),
        # https://www.gov.si/en/news/2020-04-29-prime-minister-janez-jansa-slovenia-is-currently-among-the-most-successful-countries-in-the-fight-against-the-coronavirus-epidemic-so-it-is-now-possible-to-phase-out-restrictions-for-the-foreseeable-future/
        ("2020-04-30", "further lockdown easing", "black", {"dash": "dot"}, None),
        # https://www.gov.si/en/news/2020-05-15-slovenia-declares-the-end-of-covid-19-epidemic-no-quarantine-for-eu-citizens-from-today/
        ("2020-05-15", "'epidemic state' lifted", "red", {"dash": "dot"}, None),
        # https://www.gov.si/en/news/2020-05-28-ministers-logar-and-szijjarto-agree-to-lift-restrictions-on-border-crossing/
        ("2020-05-28", "Hungary border opened", "black", {"dash": "dot"}, None),
        # https://www.total-slovenia-news.com/travel/6384-slovenia-fully-opens-borders-to-nationals-of-17-nations-today-8-june
    ],
    "Greece": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Greece#Response_measures
        # https://www.garda.com/crisis24/news-alerts/325466/greece-government-orders-nationwide-lockdown-march-23-update-11
        # https://gr.usembassy.gov/covid-19-information/
        # https://greece.greekreporter.com/2020/03/10/greece-closes-all-schools-and-universities-for-two-weeks-amid-coronavirus-fears/
        ("2020-03-10", "schools closed", "black", None, None, {'y': 0.4}),
        ("2020-03-16", "Entering 14 day quarantine; shops closed", "black", None, None),
        ("2020-03-18", "Non-EU borders closed", "red", None, None),
        ("2020-03-21", "travel to Greek islands restricted", "black", None, None),
        ("2020-03-23", "lockdown", "red", None, None),
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Greece#Lift_of_emergency_measures
        ("2020-05-04", "lockdown eased a little; masks mandatory", "black", {"dash": "dot"}, None),
        ("2020-05-11", "shops opened", "black", {"dash": "dot"}, None),
        ("2020-05-18", "schools opened", "black", {"dash": "dot"}, None),
        ("2020-06-01", "shopping malls opened", "red", {"dash": "dot"}, None),
    ],
    "Guam (US)": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Guam
        ("2020-03-16", "Public schools closed. Entering to quarantine", "black", None, None),
        ("2020-03-20", "Non-essential businesses closed", "black", None, None),
        ("2020-03-22", "State of emergency", "red", None, None),
        ("2020-04-07", "Lockdown", "red", None, None),
        # https://www.guampdn.com/story/news/local/2020/05/07/coronavirus-guam-malls-shopping-centers-set-reopen/5173906002/
        # According to Adelup's phased recovery plan, retailers, shopping centers, real estate offices,
        # automotive sales and salons are among the select businesses allowed to reopen May 11
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_Guam#May_2020
        ("2020-05-10", "restrictions mostly lifted", "black", {"dash": "dot"}, None),
        ("2020-05-29", "further restrictions eased", "black", {"dash": "dot"}, None),
    ],
    "Iowa": [
        # https://www.dailymail.co.uk/news/article-9232225/Iowa-Governor-Kim-Reynolds-rolls-COVID-19-restrictions-mask-mandate.html
        ("2021-02-06", "All restrictions lifted", "red", {"dash": "dot"}, None),
        # https://www.thegazette.com/subject/news/health/iowas-new-virus-tracking-method-sends-new-case-counts-skyrocketing-20210219
        ("2021-02-19", "Case counting method changed!", "red", {"dash": "dot"}, None),
    ],
    "Montana": [
        ("2021-02-13", "Mask mandate lifted", "blue", {"dash": "dot"}, None),
    ],
    "Sweden": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Sweden#Government_response
        ("2020-03-11", "banned >500 people gatherings", "black", None, None),
        ("2020-03-24", "bars banned", "black", None, None),
        ("2020-03-27", "banned >50 people gatherings", "black", None, None),
        ("2020-04-07", "advised against non-essential travel", "black", None, None),
        # https://www.government.se/government-policy/the-governments-work-in-response-to-the-virus-responsible-for-covid-19/
        # https://www.government.se/press-releases/2020/11/maximum-of-eight-people-permitted-at-public-gatherings-and-events/
        ("2020-11-23", "banned >8 people gatherings", "black", None, None),
        ("2021-01-18", "Law to allow stronger restrictions", "black", None, None),
    ],
    "Russia": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Russia
        ("2020-02-20", "Entry ban for Chinese", "black", None, None),
        ("2020-03-04", "More entry bans and flights cancelled", "black", None, None, {'y':0.5}),
        ("2020-03-23", "Schools closed", "black", None, None, {'y':0.75}),
        ("2020-03-27", "All flights cancelled", "black", None, None, {'y':0.6}),
        ("2020-03-30", "Borders closed; cities on lockdown", "red", None, None, {'y':0.95}),
        # https://www.themoscowtimes.com/2020/06/06/coronavirus-in-russia-the-latest-news-june-6-a69117
        # https://www.sobyanin.ru/vtoroi-etap-smyagcheniya-ogranichenii-0106
        ("2020-06-01", "Moscow restrictions eased", "black", {"dash": "dot"}, None),
    ],
    "Peru": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Peru
        ("2020-03-15", "lockdown, borders closed", "red", None, None),
        ("2020-03-18", "more strict lockdown", "red", None, None),
        # On 23 April, President Vizcarra announced the extension of quarantine and the state of emergency 2 more weeks until the 10th of May
        # https://busquedas.elperuano.pe/normaslegales/decreto-supremo-que-aprueba-la-reanudacion-de-actividades-ec-decreto-supremo-n-080-2020-pcm-1865987-1/
        # Peruvian president extends nationwide lockdown through June 30
        # https://edition.cnn.com/world/live-news/coronavirus-pandemic-05-22-20-intl/h_5d5d9a8297a9f5f61adce98ba627cf85 

        # https://pe.usembassy.gov/covid-19-information/
        # On May 25, limited economic activities deemed essential — including dentistry, ophthalmology, rehabilitation services,
        # and veterinary medicine — will be allowed to resume.
        # https://busquedas.elperuano.pe/normaslegales/decreto-supremo-que-establece-las-medidas-que-debe-observar-decreto-supremo-n-094-2020-pcm-1866708-1/
        ("2020-05-25", "Restrictions eased", "black", {"dash": "dot"}, None),

        # https://www.bloomberg.com/news/articles/2020-06-04/peru-reopens-more-industries-undeterred-by-almost-5-000-deaths
        # Peru authorized more parts of the economy to reopen
        ("2020-06-02", "Reopening - phase2", "red", {"dash": "dot"}, None),
    ],
    "UK": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_the_United_Kingdom#Government_response
        # https://en.wikipedia.org/wiki/COVID-19_pandemic_in_the_United_Kingdom#Lifting_lockdown
        # https://www.bbc.com/news/explainers-52575313
        # https://www.legislation.gov.uk/uksi/2020/327/schedule/made
        # https://www.theguardian.com/world/2020/mar/20/london-pubs-cinemas-and-gyms-may-close-in-covid-19-clampdown
        # https://www.gov.uk/government/speeches/pm-address-to-the-nation-on-coronavirus-23-march-2020
        # https://www.theguardian.com/world/2020/mar/18/coronavirus-uk-schools-to-be-closed-indefinitely-and-exams-cancelled
        # https://www.bbc.com/news/uk-51917562
        # https://eachother.org.uk/uk-coronavirus-timeline-from-liberty-to-lockdown/
        ("2020-03-16", "stay-at-home advise", "black", None, None),
        ("2020-03-18", "schools closed", "black", None, None),
        ("2020-03-21", "pubs, gyms, etc. closed", "black", None, None),
        ("2020-03-23", "partial lockdown", "red", None, None),

        # https://www.dailymail.co.uk/news/article-8302063/Garden-centres-OPEN-Customers-return-week-lockdown-eases.html
        ("2020-05-13", "restrictions eased a littled", "black", {"dash": "dot"}, None),

        # https://www.dailymail.co.uk/news/article-8366431/Boris-Johnson-says-five-tests-ease-lockdown-met.html
        # https://www.telegraph.co.uk/news/2020/06/06/end-uk-lockdown-next-phase/
        ("2020-06-01", "lockdown partially lifted", "red", {"dash": "dot"}, None),

        # protests
        # https://www.dailymail.co.uk/news/article-8394281/Priti-Patel-calls-Black-Lives-Matter-protesters-stay-home-Covid-threat.html 
#        ("2020-06-06", "huge crowds (protests)", "black", {"dash": "dot"}, None),

        # 14 day quarantine entering since June 8
        # https://sputniknews.com/uk/202005221079385391-priti-patel/

        # https://www.dailymail.co.uk/news/article-8418859/Lets-road-Rishi-Sunak-tells-Brits-shopping-normal-life.html
        # https://www.dailymail.co.uk/news/article-8421087/Stores-Britain-reopen-time-three-months-today.html
        # 06/15 malls opening
    ],
    "USA": [
        # https://www.statnews.com/2020/01/31/u-s-declares-public-health-emergency-over-coronavirus-bans-travel-from-china-by-foreign-nationals/
        # https://onemileatatime.com/china-travel-ban/
        # https://www.hhs.gov/about/news/2020/01/31/secretary-azar-declares-public-health-emergency-us-2019-novel-coronavirus.html
        ("2020-02-01", "public health emergency, flights from CN restricted", "red", None, None),
        # https://www.bbc.com/news/world-us-canada-51846923
        # https://www.telegraph.co.uk/news/2020/03/14/us-bans-flights-uk-ireland-bid-stem-coronavirus-spread/
        ("2020-03-15", "Europe flights cancelled", "black", None, None),
        # https://en.wikipedia.org/wiki/Timeline_of_the_2020_coronavirus_pandemic_in_the_United_States#cite_note-268
        ("2020-03-16", "social distance advise", "black", None, None, {'xshift': 25, 'y':0.4}),
        # https://www.foxnews.com/us/californias-coronavirus-stay-home-order-guidelines
        ("2020-03-20", "NY and California stay-at-home", "black", None, None, {'y': 0.6}),
        # https://en.wikipedia.org/wiki/U.S._state_and_local_government_response_to_the_2020_coronavirus_pandemic
        ("2020-04-03", "Stay-at-home in most of the states", "black", None, None, {'y':0.7}),
        # https://en.wikipedia.org/w/index.php?title=U.S._state_and_local_government_response_to_the_COVID-19_pandemic&oldid=955368852
        ("2020-05-04", "Stay-at-home lifted in 13 states", "black", {"dash": "dot"}, None, {'y':0.7}),
        # https://www.whitehouse.gov/openingamerica/
        # https://edition.cnn.com/us/live-news/george-floyd-protests-06-01-20
        # https://www.jdsupra.com/legalnews/bouncing-back-a-list-of-statewide-65308/
        # https://forward.ny.gov/phase-two-industries
        ("2020-05-25", "Most of the US reopened", "red", {"dash": "dot"}, None),
        ("2020-06-01", "Mass gatherings (protests)", "red", {"dash": "dot"}, None),
    ],
    "California": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_California#Government_response
        # https://www.kcra.com/article/california-newsom-declares-state-of-emergency-due-to-coronavirus/31231293
        ("2020-03-04", "State of emergency", "red", None, None),
        ("2020-03-12", "Mass gatherings banned", "black", None, None),
        ("2020-03-19", "stay-at-home", "red", None, None),
        ("2020-04-07", "LA face masks in stores", "blue", None, None),
        # https://www.gov.ca.gov/2020/05/04/governor-newsom-provides-update-on-californias-progress-toward-stage-2-reopening/
        # https://covid19.ca.gov/roadmap/#top
        # note that the stage2 is the first stage of reopening
        ("2020-05-08", "Stay-at-home eased", "black", {"dash": "dot"}, None),

        ("2020-06-01", "Mass gatherings (protests)", "red", {"dash": "dot"}, None),
    ],
    "Nebraska": [
        # https://dhhs.ne.gov/Pages/COVID-19-Directed-Health-Measures.aspx https://archive.is/Z7A9l
        # Beginning Saturday, January 30th, we are in the GREEN Zone. 
        # https://dhhs.ne.gov/Documents/DHM-Measure-Table-ENGLISH.pdf
        ("2021-01-30", "Opened up", "red", {"dash": "dot"}, None),
    ],
    "Virginia": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_Virginia#Government_responses
        # https://www.newsleader.com/story/news/local/2020/03/12/virginia-coronavirus-governor-declares-state-emergency/5034010002/
        # https://www.nbc29.com/2020/03/23/gov-northam-schools-remain-closed-through-academic-year/
        ("2020-03-12", "State of emergency", "red", None, None),
        ("2020-03-23", "Gatherings of > 10 banned; schools & public places closed", "black", None, None, {'y':0.91}),
        ("2020-03-30", "strict stay-at-home", "red", None, None),
        # https://www.governor.virginia.gov/newsroom/all-releases/2020/april/headline-856337-en.html
        ("2020-04-24", "easing announced", "black", {"dash": "dot"}, None),
        # Governor Northam said he will extend key provisions of Executive Order 53, which places restrictions on businesses 
        # and gatherings of more than 10 people, at least through next Thursday, May 14, at midnight. 

        # https://www.vdh.virginia.gov/
        # Governor Northam announced that Virginia will move into Phase One of the Forward
        # Virginia plan on Friday, May 15 (with the exception of some localities).
        ("2020-05-15", "opening-up: phase 1", "black", {"dash": "dot"}, None),
        # On March 30, Virginia Governor Ralph Northam issued a stay at home Executive Order Fifty-Three. This order will remain in effect until June 10, 2020.
    ],
    "New York": [
        # https://en.wikipedia.org/wiki/2020_coronavirus_pandemic_in_New_York_(state)#Government_response
        # https://www.nationalreview.com/2020/03/the-timeline-of-how-bill-de-blasio-prepared-new-york-city-for-the-coronavirus/
        # https://twitter.com/BilldeBlasio/status/1234648718714036229
        ("2020-03-02", "\"ignore the virus\"", "red", {"dash": "dot"}, None, {'xshift': 25}),
        ("2020-03-07", "state of emergency", "red", None, None),
        ("2020-03-17", "schools and bars closed, including NYC", "black", None, None),
        ("2020-03-20", "stay-at-home", "red", None, None),
        ("2020-04-15", "face masks required in NYC", "blue", None, None, {'y': 0.64}),

        # https://coronavirus.health.ny.gov/home
        # https://www.ny.gov/
        # "remains at pause state through May 15"
        # https://www.pressconnects.com/story/news/2020/05/11/ny-reopen-plan-phase-1-southern-tier-region-businesses-new-york-forward/3109127001/
        # New York Gov. Andrew Cuomo announced Monday that three of the state's 10 regions will be able to enter
        # "Phase 1" of a reopening process. The regions include the Finger Lakes, Mohawk Valley and the Southern Tier,
        # `which can begin the two-week process beginning Friday, May 15.
        ("2020-05-15", "Reopening starts", "red", {"dash": "dot"}, None),

        ("2020-06-01", "Mass gatherings (protests)", "red", {"dash": "dot"}, None),
        # Long Island and Mid-Hudson Valley Regions have met all seven metrics required to begin phase one of the state's regional
        # phased reopening plan; Capital Region, Western New York, Central New York, Finger Lakes, Mohawk Valley, North Country and Southern Tier can enter phase two.
        # New York City will enter phase one of reopening on June 8 adding 10 additional testing spots to hotspot areas who are predominantly low income and minority communities.

        # All regions of the state except New York City are now in Phase 1 or 2. New York City will enter phase 1 of reopening on June 8.

        # ("2020-06-06", "further opening", "blue", {"dash": "dot"}, None),
    ],
    "Wisconsin": [
        # https://www.nbcchicago.com/news/coronavirus/wisconsin-governor-issues-order-closing-businesses/2243503/
        ("2020-03-24", "Stay at home", "red", None, None),
        # https://edition.cnn.com/2020/05/13/politics/wisconsin-supreme-court-strikes-down-stay-at-home-order/index.html
        ("2020-05-14", "Stay at home lifted", "red", {"dash": "dot"}, None),
        # https://thehill.com/homenews/state-watch/509824-wisconsin-governor-issues-statewide-mask-mandate
        ("2020-07-30", "Masks mandatory", "blue", None, None),
    ],
    "Mississippi": [
        # https://www.theepochtimes.com/mississippi-to-lift-mask-mandate-allow-businesses-back-at-full-capacity_3718207.html?utm_source=gab
        ("2021-03-03", "Lockdown and mask mandate ended", "red", {"dash": "dot"}, None),
    ],
    "Texas": [
        # https://open.texas.gov/uploads/files/organization/opentexas/EO-GA-34-opening-Texas-response-to-COVID-disaster-IMAGE-03-02-2021.pdf
        ("2021-03-03", "Lockdown and mask mandate ended", "red", {"dash": "dot"}, None),
    ],
}

os.makedirs("per_country/", exist_ok=True)
for our_country in countries:
    print(our_country)
    days = 1
    prev_count = 0
    prev_recovered = 0
    prev_death_toll = 0
    results = []
    for fn in fns+fns_italy_regions:
        if fn in fns_italy_regions:
            f = open(ITALY_GIT+"/dati-regioni/"+fn)
            if not our_country.endswith("(Italy region)"): continue
        else:
            f = open(fn)
            if our_country.endswith("(Italy region)"): continue
        f_csv = csv.reader(f)
        header = next(f_csv)
        style = "normal"

        if fn in fns_italy_regions:
            style = "italy_region"
        else:
            if "FIPS" in header[0]:
                style = "fips"
        if style == "italy_region":
            date_str = fn.partition('.')[0].rpartition("-")[2]
            year = date_str[:4]
            month = date_str[4:6]
            day = date_str[6:8]
        else:
            month, day, year = fn.partition('.')[0].split("-")
        current_date = f"{year}-{month}-{day}"
        cfr = 0
        # CFR projected guids for CFR if 4% of confirmed cases before 11 days died
        # 11 day lag beacuse: "time from illness onset to death 18.5days"
        #   (https://www.thelancet.com/action/showPdf?pii=S0140-6736%2820%2930566-3
        #   - about 4 days from onset to symptoms (median) - 2 day until positive test
        # expressed as would-be CFR for today
        cfr_projected_04 = 0
        cfr_last_21d = 0
        cfr_recovered_only = 0
        count = 0
        death_toll = 0
        death_toll_projected_04 = 0
        recovered = 0
        have = False

        for line in f_csv:
            if style == "normal":
                region = line[0]
                country = line[1]
            elif style == "fips":
                region = line[2]
                country = line[3]
            elif style == "italy_region":
                region = ""
                country = line[3]+" (Italy region)"
            else:
                raise ValueError("invalid style")

            is_our_country = False
            if region:
                if isinstance(our_country, tuple) or isinstance(our_country, list):
                    if (region, country) == our_country:
                        is_our_country = True
                if our_country in ALTERNATIVE_NAMES:
                    for our_country_alt in ALTERNATIVE_NAMES[our_country]:
                        if isinstance(our_country_alt, tuple) or isinstance(our_country_alt, list):
                            if (region, country) == our_country_alt:
                                is_our_country = True
            else:
                if country == our_country or (our_country in ALTERNATIVE_NAMES
                        and country in ALTERNATIVE_NAMES[our_country]):
                    is_our_country = True

            if is_our_country:
                have = True
                if style == "normal":
                    if line[3]:
                        count += int(float(line[3]))
                    if line[4]:
                        death_toll += int(float(line[4]))
                    if line[5]:
                        recovered += int(float(line[5]))
                elif style == "fips":
                    if line[7]:
                        count += int(float(line[7]))
                    if line[8]:
                        death_toll += int(float(line[8]))
                    if line[9]:
                        recovered += int(float(line[9]))
                elif style == "italy_region":
                    recovered += int(float(line[13]))
                    count += int(float(line[17]))
                    death_toll += int(float(line[14]))
                else:
                    raise ValueError("invalid style")

        # cfr
        if count > 0:
            cfr = death_toll / float(count)
            if recovered + death_toll == 0:
                # division by zero -> show cfr as 0
                cfr_recovered_only = 0
            else:
                cfr_recovered_only = death_toll / float(recovered + death_toll)
            # cfr_projected_04 and death_toll_projected_04
            if len(results) >= 11:
                result_11d_old = results[-11]
                cfr_projected_04 = (result_11d_old['total_cases'] * 0.04) / float(count)
                death_toll_projected_04 = result_11d_old['total_cases'] * 0.04
            if len(results) >= 21:
                new_deaths_last_21d = sum(r['new_deaths'] for r in results[-21:])
                new_cases_last_21d = sum(r['new_cases'] for r in results[-21:])
                # division by zero
                if new_cases_last_21d == 0:
                    if new_deaths_last_21d >= 3:
                        # no new cases in last 21 days but >=3 deaths in last 21 days 
                        # super suspicious -> show last 21d CFR as 100% as a warning
                        cfr_last_21_d = 100
                    else:
                        # below the warning level so just do not show last 21d CFR now
                        cfr_last_21_d = 0
                else:
                    cfr_last_21d = new_deaths_last_21d / float(new_cases_last_21d)
        if have and count > 0:
            # day,date,new_cases,new_cases_per_million,total_cases,total_cases_per_million
            population = float(POPULATION[our_country] / float(1e6))  # population in millions
            result = {
                'days': days,
                'current_date': current_date,
                'new_cases': count-prev_count,
                'new_cases_per_million': (count-prev_count) / population,
                'total_cases': count,
                'new_recovered': recovered-prev_recovered,
                'new_recovered_per_million': (recovered-prev_recovered) / population,
                'total_recovered': recovered,
                'total_recovered_per_million': recovered / population,
                'total_cases_per_million': count/population,
                'new_deaths': death_toll-prev_death_toll,
                'new_deaths_per_million': (death_toll-prev_death_toll) / population,
                'total_deaths': death_toll,
                'total_deaths_per_million': death_toll/population,
                'total_deaths_population_ratio': death_toll/float(POPULATION[our_country]),
                'total_deaths_projected_04': death_toll_projected_04,
                'total_deaths_per_million_projected_04': death_toll_projected_04/population,
                'cfr': cfr,
                'cfr_recovered_only': cfr_recovered_only,
                'cfr_last_21d': cfr_last_21d,
                'cfr_projected_04': cfr_projected_04,
            }
            results.append(result)

            prev_count = count
            prev_death_toll = death_toll
            prev_recovered = recovered
            days += 1

        if not have and days > 1:
            print(f"!! Missing {our_country} in {fn}")
            result = result.copy()
            result["days"] += 1
            # date += 1
            y,m,d = result["current_date"].split("-")
            new_date = datetime.date(int(y),int(m),int(d)) + datetime.timedelta(days=1)
            result["current_date"] = f"{new_date.year:4d}-{new_date.month:02d}-{new_date.day:02d}"
            results.append(result)
            #
            days += 1

        # calculate (R)ate value
        for i, result in enumerate(results):
            # R7 (daily cases compared to 7 days from now, 7 days moving average)
            # we need previous 3 values and next 10 (3+7) values to calculate rate
            if i >= 3 and i < len(results) - 10:
                try:
                    result['rate'] = float(sum(n['new_cases'] for n in results[i+4:i+10+1]) / 7) / float(sum(n['new_cases'] for n in results[i-3:i+4]) / 7 )
                    # invalid (one of the new_cases must be invalid) -> makes no sense to calculate rate -> set none
                    if result['rate'] < 0:
                        del result['rate']
                except ZeroDivisionError:
                    if sum(n['new_cases'] for n in results[i+4:i+11]) == 0:
                        result['rate'] = 0
                    else:
                        result['rate'] = float('inf')

            if rate_cap:
                if 'rate' in result and result['rate'] > rate_cap:
                    result["rate"] = rate_cap

        # calculate 7-day moving average of new cases and new death toll
        for i, result in enumerate(results):
            # we need prev and next 3 values to calculate moving average
            if i < 3: continue
            if i > len(results) - 4: continue

            result['new_cases_per_million_7_day_moving_average'] = float(sum(n['new_cases_per_million'] for n in results[i-3:i+4]) / 7 )
            result['new_cases_7_day_moving_average'] = float(sum(n['new_cases'] for n in results[i-3:i+4]) / 7 )
            result['new_deaths_per_million_7_day_moving_average'] = float(sum(n['new_deaths_per_million'] for n in results[i-3:i+4]) / 7 )
            result['new_deaths_7_day_moving_average'] = float(sum(n['new_deaths'] for n in results[i-3:i+4]) / 7 )
            result['new_recovered_per_million_7_day_moving_average'] = float(sum(n['new_recovered_per_million'] for n in results[i-3:i+4]) / 7 )
            result['new_recovered_7_day_moving_average'] = float(sum(n['new_recovered'] for n in results[i-3:i+4]) / 7 )

    # dump to CSV
    display_name = DISPLAY_NAME.get(our_country, our_country)
    results_f = open(f"per_country/{display_name}.csv", "w")
    results_f.write("day,date,new_cases,new_cases_per_million,total_cases,total_cases_per_million,new_cases_per_million_7d_mavg,"
            "new_cases_7d_mavg,new_deaths,new_deaths_per_million,total_deaths,total_deaths_per_million,total_deaths_population_ratio,"
            "new_deaths_per_million_7d_mavg,new_deaths_7d_mavg,"
            "cfr,cfr_recovered_only,cfr_last_21d,cfr_project_04,"
            "total_deaths_projected_04,total_deaths_per_million_projected_04,new_recovered,total_recovered,"
            "total_recovered_per_million,new_recovered_per_million,new_recovered_per_million_7d_mavg,new_recovered_7d_mavg,rate\n")

    for result in results:
        if 'rate' in result:
            rate = f"{result['rate']:.3f}"
        else:
            rate = ""

        if 'new_cases_per_million_7_day_moving_average' in result:
            new_cases_7d = f"{result['new_cases_per_million_7_day_moving_average']:.2f}"
        else:
            new_cases_7d = ''
        if 'new_deaths_per_million_7_day_moving_average' in result:
            new_deaths_7d = f"{result['new_deaths_per_million_7_day_moving_average']:.4f}"
        else:
            new_deaths_7d = ''
        if 'new_cases_7_day_moving_average' in result:
            new_cases_7d_abs = f"{result['new_cases_7_day_moving_average']:.2f}"
        else:
            new_cases_7d_abs = ''
        if 'new_deaths_7_day_moving_average' in result:
            new_deaths_7d_abs = f"{result['new_deaths_7_day_moving_average']:.4f}"
        else:
            new_deaths_7d_abs = ''
        if 'new_recovered_per_million_7_day_moving_average' in result:
            new_recovered_7d = f"{result['new_recovered_per_million_7_day_moving_average']:.2f}"
        else:
            new_recovered_7d = ''
        if 'new_recovered_7_day_moving_average' in result:
            new_recovered_7d_abs = f"{result['new_recovered_7_day_moving_average']:.2f}"
        else:
            new_recovered_7d_abs = ""

        results_f.write(f"{result['days']},{result['current_date']},{result['new_cases']},{result['new_cases_per_million']:.2f},"
                        f"{result['total_cases']},{result['total_cases_per_million']:.2f},"
                        f"{new_cases_7d},{new_cases_7d_abs},{result['new_deaths']},"
                        f"{result['new_deaths_per_million']:.4f},{result['total_deaths']},{result['total_deaths_per_million']:.4f},"
                        f"{result['total_deaths_population_ratio']:.7f},{new_deaths_7d},{new_deaths_7d_abs},"
                        f"{result['cfr']:.6f},{result['cfr_recovered_only']:.6f},{result['cfr_last_21d']:.6f},"
                        f"{result['cfr_projected_04']:.7f},{result['total_deaths_projected_04']:.7f},"
                        f"{result['total_deaths_per_million_projected_04']:.7f},"
                        f"{result['new_recovered']},{result['total_recovered']},{result['total_recovered_per_million']},"
                        f"{result['new_recovered_per_million']:.2f},{new_recovered_7d},{new_recovered_7d_abs},"
                        f"{rate}\n")

    # graph
    print("..generating graphs")
    # graph: new daily cases per million + rate
    fig_cases = go.Figure()

    rate_first_i = None
    for i, result in enumerate(results):
        if ("new_cases_per_million_7_day_moving_average" in result and
                result["new_cases_per_million_7_day_moving_average"] >= rate_cutoff):
            rate_first_i = i
            break
    rate_x = []
    rate_y = []
    if rate_first_i is not None:
        rate_results = [r for i,r in enumerate(results) if 'rate' in r and i >= rate_first_i]
        rate_x = [r['current_date'].replace("-", "/") for r in rate_results]
        rate_y = [r['rate'] for r in rate_results]

    if our_country in TEMPERATURES:
        temps = {}
        with open(f"temps/{TEMPERATURES[our_country]}.csv", "r") as f:
            temps_r = csv.reader(f)
            next(temps_r)
            for line in temps_r:
                m = line[0][4:6]
                d = line[0][6:8]
                m = m[1:]  # when date is fixed to [5:] instead of [6:] remove this line :D
                temps[f"{m}/{d}"] = line[1]

        temps_x = [n['current_date'].replace("-", "/") for n in results]
        temps_y = []
        for t_ in temps_x:
            if t_ in temps:
                temps_y.append(float(temps[t_]))
            else:
                temps_y.append(None)

        fig_cases.add_traces([
            go.Scatter(
                x=temps_x,
                y=temps_y,
                marker={
#                    "color": temps_y,
                    "color": "blue",
                    'size': 2,
#                    'cmin': min(temps_y),
#                    'cmax': max(temps_y),
#                    'colorscale': "thermal",
                },
                yaxis='y3',
                mode='lines',
                opacity=0.2,
                name='Daily average temperature',
            ),
        ])

    fig_cases.add_traces([
            # new cases 
            go.Bar(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['new_cases_per_million'] for n in results],
                marker={"color": "grey", "opacity": 0.7},
                name='daily new cases',
            ),
            # total cases
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['total_cases_per_million'] for n in results],
                marker={"color": "blue", 'size': 2},
                name='total confirmed cases',
                visible='legendonly',
            ),
            # new cases - moving average
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_cases_per_million_7_day_moving_average' in n],
                y=[n['new_cases_per_million_7_day_moving_average'] for n in results if 'new_cases_per_million_7_day_moving_average' in n],
                marker={"color": "grey", 'size': 3},
                yaxis='y1',
                name='daily new cases (7-day moving average)',
            ),
            # R7 rate
            go.Scatter(
                x=rate_x,
                y=rate_y,
                marker={"color": "crimson", 'size': 2},
                yaxis='y2',
                name='(R7) 7-day rate',
            ),
            ])


    fig_cases.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        },
        margin={'l': 30, 'r': 140, 'pad':4},

        yaxis = dict(
            title = {
                'text': "new daily cases per one million<br>(solid line - 7 day moving average)",
                'font': {'size': 18}
            },
            color="grey",
            side='right',
            rangemode='tozero',
            tickprefix="  ",
        ),

        yaxis2 = dict(
            title = {
                'text': "(R)ate of change - 7-day rate <br>"
                        "calculated as today+7 days/today using moving average of daily new cases<br>"
                        f"(calculated only after the moving average has reached {rate_cutoff})",
                'font': {'size': 14},
            },
            color="crimson",
            rangemode='tozero',
            showgrid=False,
            ticksuffix="  ",
            # fixedrange=True,
            range=(0,5.05),
            side="left",
            overlaying="y",
        ),

    )

    if our_country in TEMPERATURES:
        fig_cases.update_layout(
            yaxis3 = dict(
                title = {
                    'text': "Daily average temperature",
                    'font': {'size': 14},
                },
                color="lightblue",
                showgrid=False,
                ticksuffix="  ",
                fixedrange=True,
                range=[-5,30],
                side="right",
                overlaying="y",
            )
        )

    fig_cases.update_xaxes(
        title= {'text': "date", 'font': {'size': 20}},
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
    )


    # dashed grid
    # draw yaxis lines from 0 to 20
    for grid_val in [x/10.0 for x in range(0,210,5)]:
        if grid_val % 1 == 0:
            opacity = 0.8
        else:
            opacity = 0.3
        fig_cases.add_shape(
            dict(
                type="line",
                yref="y2",
                xref="paper",
                x0=0,
                y0=grid_val,
                x1=1,
                #x1=0.939,  # eh
                y1=grid_val,
                line=dict(
                    color="crimson",
                    width=1,
                    dash="dash",
                ),
                opacity=opacity,
                layer="below",
        ))

    # add Vertical lines for events
    if our_country in EVENTS:
        for event in EVENTS[our_country]:
            d = event[0]
            label = event[1]
            x_pos = d.replace("-", "/")
            if len(event) >= 3 and event[2] is not None:
                color = event[2]
            else:
                color = "black"

            line = {
                'width': 4,
                'dash': 'dash',
                'color': color,
            }

            font = {'size': 18, 'color': color}

            annotation = {
                'xanchor': "right",
                'font': font,
                'y': 0.9,
                'textangle': 270,
                'xshift': -10,
            }

            if len(event) >= 4 and event[3] is not None:
                line.update(event[3])

            if len(event) >= 5 and event[4] is not None:
                font.update(event[4])

            if len(event) >= 6 and event[5] is not None:
                annotation.update(event[5])

            fig_cases.add_shape(
                dict(
                    type="line",
                    yref="paper",
                    x0=x_pos,
                    y0=0,
                    y1=1,
                    x1=x_pos,
                    line=line,
                    opacity=1,
                    layer="above",
            ))

            fig_cases.add_annotation(
                x=x_pos,
                yref="paper",
                text=label,
                **annotation
            )

    # graph: new daily death toll + moving average + total death toll (per million)
    fig_mortality = go.Figure()

    fig_mortality.add_traces([
            # new death toll
            go.Bar(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['new_deaths_per_million'] for n in results],
                marker={"color": "grey", "opacity": 0.7},
                name='daily new death toll',
            ),
            # new death toll - moving average
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_deaths_per_million_7_day_moving_average' in n],
                y=[n['new_deaths_per_million_7_day_moving_average'] for n in results if 'new_deaths_per_million_7_day_moving_average' in n],
                marker={"color": "black", 'size': 3},
                name='daily new death toll (7-day moving average)',
            ),
            # total death toll
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['total_deaths_per_million'] for n in results],
                marker={"color": "blue", 'size': 3},
                name='total death toll',
                visible='legendonly',
            ),
            # total death toll (per million) projected 4% 11-day lag
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['total_deaths_per_million_projected_04'] for n in results],
                line={"width": 3, "color": "rgba(0,0, 250, 0.4)", 'dash': 'dot'},
                name='Projected 11-day lag 4% CFR deaths',
                visible='legendonly',
            ),
            ])

    fig_mortality.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        margin={'l': 30, 'r': 140},
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        }
    )

    fig_mortality.update_xaxes(
        title= {
            'text': "date",
            'font': {'size': 20},
        },
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
    )

    fig_mortality.update_yaxes(
        title = {
            'text': "new daily death toll per one million<br>(solid line - 7 day moving average)",
            'font': {'size': 18},
        },
        color="grey",
        side='right',
        rangemode='tozero',
        tickprefix="  "
    )

    # graph: absolute values for new and total daily cases (+moving avg of course)
    fig_cases_absolute = go.Figure()

    fig_cases_absolute.add_traces([
        # new cases
        go.Bar(
            x=[n['current_date'].replace("-", "/") for n in results],
            y=[n['new_cases'] for n in results],
            marker={"color": "grey", "opacity": 0.7},
            name='daily new cases',
        ),
        # new cases - moving average
        go.Scatter(
            x=[n['current_date'].replace("-", "/") for n in results if 'new_cases_7_day_moving_average' in n],
            y=[n['new_cases_7_day_moving_average'] for n in results if 'new_cases_7_day_moving_average' in n],
            marker={"color": "#9932CC", 'size': 3},
            name='daily new cases (7-day moving average)',
        ),
        # total cases
        go.Scatter(
            x=[n['current_date'].replace("-", "/") for n in results],
            y=[n['total_cases'] for n in results if 'total_cases' in n],
            marker={"color": "orange", 'size': 3},
            name='total cases',
            visible='legendonly',
        ),
    ])

    fig_cases_absolute.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        margin={'l': 30, 'r': 140},
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        }
    )

    fig_cases_absolute.update_xaxes(
        title = {
            'text': "date",
            "font": {'size': 20},
        },
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
    )

    fig_cases_absolute.update_yaxes(
        title = {
            'text': "daily new cases<br>(solid line - 7 day moving average)",
            'font': {'size': 18},
        },
        color="grey",
        side='right',
        rangemode='tozero',
        tickprefix="  "
    )

    # graph: absolute values for new and total death toll (+moving avg of course)
    fig_mortality_absolute = go.Figure()

    fig_mortality_absolute.add_traces([
        # new deaths
        go.Bar(
            x=[n['current_date'].replace("-", "/") for n in results],
            y=[n['new_deaths'] for n in results],
            marker={"color": "grey", "opacity": 0.7},
            name='daily new death toll',
        ),
        # new deaths - moving average
        go.Scatter(
            x=[n['current_date'].replace("-", "/") for n in results if 'new_deaths_7_day_moving_average' in n],
            y=[n['new_deaths_7_day_moving_average'] for n in results if 'new_deaths_7_day_moving_average' in n],
            marker={"color": "black", 'size': 3},
            name='daily new death toll (7-day moving average)',
        ),
        # total deaths
        go.Scatter(
            x=[n['current_date'].replace("-", "/") for n in results],
            y=[n['total_deaths'] for n in results if 'total_deaths' in n],
            marker={"color": "blue", 'size': 3},
            name='total death toll',
            visible='legendonly',
        ),
        # total death toll projected 4% 11-day lag
        go.Scatter(
            x=[n['current_date'].replace("-", "/") for n in results],
            y=[n['total_deaths_projected_04'] for n in results],
            line={"width": 3, "color": "rgba(0,0, 250, 0.4)", 'dash': 'dot'},
            name='Projected 11-day lag 4% CFR deaths',
            visible='legendonly',
        ),
    ])

    fig_mortality_absolute.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        margin={'l': 30, 'r': 140},
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        },
    )

    fig_mortality_absolute.update_xaxes(
        title= {'text': "date", 'font': {'size': 20}},
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
    )

    fig_mortality_absolute.update_yaxes(
        title = {
            'text': "daily death toll<br>(solid line - 7 day moving average)",
            'font': {'size': 18},
        },
        color="grey",
        side='right',
        rangemode='tozero',
        tickprefix="  ",
    )

    # graph: new daily recovered + moving average + total recovered (per million)
    fig_recovery = go.Figure()

    fig_recovery.add_traces([
            # new daily recovered
            go.Bar(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['new_recovered_per_million'] for n in results],
                marker={"color": "grey", "opacity": 0.7},
                name='daily new recoveries',
            ),
            # new daily recovered - moving average
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_recovered_per_million_7_day_moving_average' in n],
                y=[n['new_recovered_per_million_7_day_moving_average'] for n in results if 'new_recovered_per_million_7_day_moving_average' in n],
                marker={"color": "rgb(5, 81, 10)", 'size': 3},
                name='daily new recoveries (7-day moving average)',
            ),
            # total recoveries
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['total_recovered_per_million'] for n in results],
                marker={"color": "rgb(4, 122, 149)", 'size': 3},
                name='total recoveries',
                visible='legendonly',
            ),
            ])

    fig_recovery.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        margin={'l': 30, 'r': 140},
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        }
    )

    fig_recovery.update_xaxes(
        title= {
            'text': "date",
            'font': {'size': 20},
        },
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
    )

    fig_recovery.update_yaxes(
        title = {
            'text': "new daily recoveries per one million<br>(solid line - 7 day moving average)",
            'font': {'size': 18},
        },
        color="grey",
        side='right',
        rangemode='tozero',
        tickprefix="  "
    )

    # graph: new daily recovered + moving average + total recovered (absolute numbers)
    fig_recovery_absolute = go.Figure()

    fig_recovery_absolute.add_traces([
            # new daily recovered
            go.Bar(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['new_recovered'] for n in results],
                marker={"color": "grey", "opacity": 0.7},
                name='daily new recoveries',
            ),
            # new daily recovered - moving average
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_recovered_7_day_moving_average' in n],
                y=[n['new_recovered_7_day_moving_average'] for n in results if 'new_recovered_7_day_moving_average' in n],
                marker={"color": "rgb(5, 81, 10)", 'size': 3},
                name='daily new recoveries (7-day moving average)',
            ),
            # total recoveries
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['total_recovered'] for n in results],
                marker={"color": "rgb(4, 122, 149)", 'size': 3},
                name='total recoveries',
                visible='legendonly',
            ),
            ])

    fig_recovery_absolute.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        margin={'l': 30, 'r': 140},
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        }
    )

    fig_recovery_absolute.update_xaxes(
        title= {
            'text': "date",
            'font': {'size': 20},
        },
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
    )

    fig_recovery_absolute.update_yaxes(
        title = {
            'text': "new daily recoveries<br>(solid line - 7 day moving average)",
            'font': {'size': 18},
        },
        color="grey",
        side='right',
        rangemode='tozero',
        tickprefix="  "
    )

    # graph: CFR, CFR 12 day lag, 12 day-lag projected CFRs (legend only),
    # death toll as percents of population (legend only)
    fig_cfr = go.Figure()

    fig_cfr.add_traces([
            # CFR
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['cfr']*100 for n in results],
                marker={"color": "black", 'size': 3},
                name='CFR (total death toll/total cases)',
            ),
            # projected CFR if 4% of total cases 11 days ago died
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['cfr_projected_04']*100 for n in results],
                line={"width": 3, "color": "rgba(0, 0, 0, 0.4)", 'dash': 'dot'},
                name='Projected CFR of 4% CFR with 11 day lag',
            ),
            # CFR - based on deaths/deaths+recovered
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['cfr_recovered_only']*100 for n in results],
                line={"width": 3, "color": "#b997ef"},
                name='CFR recovered only (total death toll/deaths+recovered)',
                visible='legendonly',
            ),
            # CFR for last 21 days (new deaths in last 21 days/new cases in last 21 days)
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['cfr_last_21d']*100 for n in results],
                line={"width": 3, "color": "rgba(213, 80, 66, 0.4)"},
                name='Last 21 days CFR (deaths/cases in last 21 days)',
                visible='legendonly',
            ),
            # total death toll as % of population
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results],
                y=[n['total_deaths_population_ratio']*100 for n in results],
                line={"width": 3, "color": "rgba(220, 0, 0, 0.8)"},
                name='total death toll (% of population)',
                visible='legendonly',
            ),
            # daily death toll (y2)
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_deaths_per_million_7_day_moving_average' in n],
                y=[n['new_deaths_per_million_7_day_moving_average'] for n in results if 'new_deaths_per_million_7_day_moving_average' in n],
                marker={"color": "crimson", 'size': 2},
                yaxis='y2',
                name='New deaths (7-day moving average, per million)',
                visible='legendonly',
            ),
            # daily cases (y3)
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_cases_per_million_7_day_moving_average' in n],
                y=[n['new_cases_per_million_7_day_moving_average'] for n in results if 'new_cases_per_million_7_day_moving_average' in n],
                marker={"color": "orange", 'size': 2},
                yaxis='y3',
                name='New cases (7-day moving average, per million)',
                visible='legendonly',
            ),
            # daily recovered (y4)
            go.Scatter(
                x=[n['current_date'].replace("-", "/") for n in results if 'new_recovered_per_million_7_day_moving_average' in n],
                y=[n['new_recovered_per_million_7_day_moving_average'] for n in results if 'new_recovered_per_million_7_day_moving_average' in n],
                marker={"color": "green", 'size': 2},
                yaxis='y4',
                name='New recovered (7-day moving average, per million)',
                visible='legendonly',
            ),
            ])

    fig_cfr.update_layout(
        title = {
            'text': f"{display_name}",
            'xanchor': 'center',
            'x': 0.5,
            'font': {'size': 30},
        },
        margin={'l': 30, 'r': 140, 'pad':4},
        showlegend=True,
        legend = {
            "orientation": "h",
            "y": 1.05,
        }
    )

    fig_cfr.update_xaxes(
        title= {'text': "date", 'font': {'size': 20}},
        tickangle=80,
        tickmode = "array",
        ticktext = [n['current_date'].replace("-", "/")+f" - day {i}" for i,n in enumerate(results, 1)],
        tickvals = [n['current_date'].replace("-", "/") for n in results],
        side = 'bottom',
        domain = [0.06, 1],
    )

    fig_cfr.update_layout(
        yaxis = dict(
            title = {
                'text': "%",
                'font': {'size': 18},
            },
            color="grey",
            side='right',
            rangemode='tozero',
            tickprefix="  "
        ),
        yaxis2 = dict(
            title = {
                'text': "New deaths (7-day moving average, per million)",
                'font': {'size': 12},
            },
            tickfont = {'size': 12},
            color="crimson",
            ticksuffix="  ",
            side="left",
            anchor="free",
            overlaying="y",
        ),
        yaxis3 = dict(
            title = {
                'text': "New cases (7-day moving average, per million)",
                'font': {'size': 12},
            },
            tickfont = {'size': 12},
            color="orange",
            ticksuffix="  ",
            side="left",
            anchor="free",
            overlaying="y",
            position=0.03,
        ),
        yaxis4 = dict(
            title = {
                'text': "New recovered (7-day moving average, per million)",
                'font': {'size': 12},
            },
            tickfont = {'size': 12},
            color="green",
            ticksuffix="  ",
            side="left",
            anchor="free",
            overlaying="y",
            position=0.06,
        )
    )


    # plot the graphs
    offline.plot(fig_cases, filename=f"per_country/new_cases_{display_name}.html", auto_open=False)
    offline.plot(fig_mortality, filename=f"per_country/death_toll_{display_name}.html", auto_open=False)
    offline.plot(fig_cases_absolute, filename=f"per_country/reported_cases_{display_name}.html", auto_open=False)
    offline.plot(fig_mortality_absolute, filename=f"per_country/reported_deaths_{display_name}.html", auto_open=False)
    offline.plot(fig_recovery, filename=f"per_country/recovered_{display_name}.html", auto_open=False)
    offline.plot(fig_recovery_absolute, filename=f"per_country/reported_recovered_{display_name}.html", auto_open=False)
    offline.plot(fig_cfr, filename=f"per_country/cfr_{display_name}.html", auto_open=False)
